;;;
;;;Part of: Vicare
;;;Contents: platform constants
;;;Date: Mon Nov  7, 2011
;;;
;;;Abstract
;;;
;;;	Export an  identifier syntax  for each platform  constant.  This
;;;	library can be used also by Vicare's source code.
;;;
;;;Copyright (C) 2011, 2012, 2013, 2015 Marco Maggi <marco.maggi-ipsu@poste.it>
;;;
;;;This program is free software:  you can redistribute it and/or modify
;;;it under the terms of the  GNU General Public License as published by
;;;the Free Software Foundation, either version 3 of the License, or (at
;;;your option) any later version.
;;;
;;;This program is  distributed in the hope that it  will be useful, but
;;;WITHOUT  ANY   WARRANTY;  without   even  the  implied   warranty  of
;;;MERCHANTABILITY  or FITNESS FOR  A PARTICULAR  PURPOSE.  See  the GNU
;;;General Public License for more details.
;;;
;;;You should  have received  a copy of  the GNU General  Public License
;;;along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;;


(library (vicare platform constants (0 4 2015 6 18))
  (export

;;; data type sizes and limits

    SIZEOF_CHAR		SIZEOF_SHORT
    SIZEOF_INT		SIZEOF_LONG
    SIZEOF_LONG_LONG
    SIZEOF_SIZE_T	SIZEOF_SSIZE_T
    SIZE_T_MAX
    SSIZE_T_MAX		SSIZE_T_MIN
    SIZEOF_OFF_T
    OFF_T_MAX		OFF_T_MIN

    CHAR_BIT		SCHAR_MIN	SCHAR_MAX
    UCHAR_MAX		CHAR_MIN	CHAR_MAX
    SHRT_MIN		SHRT_MAX	USHRT_MAX
    INT_MIN		INT_MAX		UINT_MAX
    LONG_MIN		LONG_MAX	ULONG_MAX
    LONG_LONG_MIN	LONG_LONG_MAX	ULONG_LONG_MAX
    WCHAR_MAX

;;; miscellaneous platform constants

    ;; constants for waitpid and similar
    WAIT_ANY		WAIT_MYPGRP	WCONTINUED
    WEXITED		WNOHANG		WNOWAIT
    WSTOPPED		WUNTRACED
    P_PID		P_PGID		P_ALL
    CLD_CONTINUED	CLD_DUMPED	CLD_EXITED
    CLD_KILLED		CLD_STOPPED	CLD_TRAPPED

;;;These are not fixnums.
;;;
;;;__WCLONE		__WALL		__WNOTHREAD

;;; constants for file system inspection
    S_IFDIR		S_IFCHR		S_IFBLK
    S_IFREG		S_IFLNK		S_IFSOCK
    S_IFIFO

;;; seek whence arguments
    SEEK_SET SEEK_CUR SEEK_END

;;; file descriptor related flags
    O_ACCMODE		O_APPEND	O_ASYNC
    O_CREAT		O_EXCL		O_EXEC
    O_EXLOCK		O_FSYNC		O_IGNORE_CTTY
    O_NDELAY		O_NOCTTY	O_NOLINK
    O_NONBLOCK		O_NOTRANS	O_RDONLY
    O_RDWR		O_READ		O_SHLOCK
    O_SYNC		O_TRUNC		O_WRITE
    O_WRONLY

;;; This is GNU specific.
    O_NOATIME

;;; These are Linux specific.
    O_CLOEXEC		O_DIRECT	O_DIRECTORY
    O_LARGEFILE		O_NOFOLLOW	FD_CLOEXEC

    EPOLL_CLOEXEC	EPOLL_CTL_ADD	EPOLL_CTL_MOD
    EPOLL_CTL_DEL
    EPOLLIN		EPOLLOUT	EPOLLRDHUP
    EPOLLPRI		EPOLLERR	EPOLLHUP
    EPOLLET		EPOLLONESHOT

;;; ioctl action selection
    F_DUPFD		F_GETFD		F_GETFL
    F_GETLK		F_GETOWN	F_SETFD
    F_SETFL		F_SETLKW	F_SETLK
    F_SETOWN		F_RDLCK		F_UNLCK
    F_WRLCK

;;; miscellaneous file-related constants
    R_OK		W_OK		X_OK
    F_OK

;;; miscellaneous constants
    L_ctermid		L_tmpnam	L_cuserid
    EOF			MAXSYMLINKS	MAXPATHLEN
    CLOCKS_PER_SEC	RAND_MAX

;;; mode bits
    S_IRUSR		S_IWUSR		S_IXUSR
    S_IRGRP		S_IWGRP		S_IXGRP
    S_IROTH		S_IWOTH		S_IXOTH
    S_IRWXU		S_IRWXG		S_IRWXO
    S_ISUID		S_ISGID		S_ISVTX

;;; dirent stuff
    DT_BLK		DT_CHR		DT_DIR
    DT_FIFO		DT_LNK		DT_REG
    DT_SOCK		DT_UNKNOWN

;;; constants related to ftw() and nftw()
    FTW_F		FTW_D		FTW_NS
    FTW_DNR		FTW_SL		FTW_DP
    FTW_SLN		FTW_PHYS	FTW_MOUNT
    FTW_CHDIR		FTW_DEPTH	FTW_ACTIONRETVAL
    FTW_CONTINUE	FTW_STOP	FTW_SKIP_SUBTREE
    FTW_SKIP_SIBLINGS

;;; setitimer() stuff
    ITIMER_REAL		ITIMER_VIRTUAL	ITIMER_PROF

;;; mmap constants
    MAP_PRIVATE		MAP_SHARED	MAP_FIXED
    MAP_ANON		MAP_ANONYMOUS	MAP_32BITS
    MAP_DENYWRITE	MAP_EXECUTABLE	MAP_FILE
    MAP_GROWSDOWN	MAP_LOCKED	MAP_NONBLOCK
    MAP_NORESERVE	MAP_POPULATE	MAP_STACK
    MAP_AUTOGROW	MAP_AUTORESRV	MAP_COPY
    PROT_READ		PROT_WRITE	PROT_EXEC
    PROT_NONE		MS_SYNC		MS_ASYNC
    MREMAP_MAYMOVE

    MADV_NORMAL		MADV_RANDOM	MADV_SEQUENTIAL
    MADV_WILLNEED	MADV_DONTNEED
    POSIX_MADV_NORMAL		POSIX_MADV_RANDOM
    POSIX_MADV_SEQUENTIAL	POSIX_MADV_WILLNEED
    POSIX_MADV_DONTNEED
    MCL_CURRENT		MCL_FUTURE

;;; select related symbols
    FD_SETSIZE

;;;; for lockf
    F_LOCK		F_TLOCK
    F_ULOCK		F_TEST

;;; poll related symbols
    POLLIN		POLLPRI		POLLOUT
    POLLRDHUP		POLLERR		POLLHUP
    POLLNVAL

;;; system capacity limitations
    ARG_MAX		_POSIX_ARG_MAX
    CHILD_MAX		_POSIX_CHILD_MAX
    OPEN_MAX		_POSIX_OPEN_MAX
    STREAM_MAX		_POSIX_STREAM_MAX
    TZNAME_MAX		_POSIX_TZNAME_MAX
    NGROUPS_MAX		SSIZE_MAX	RE_DUP_MAX

;;; overall system options
    _POSIX_JOB_CONTROL	_POSIX_SAVED_IDS
    _POSIX2_C_DEV	_POSIX2_FORT_DEV
    _POSIX2_FORT_RUN	_POSIX2_LOCALEDEF
    _POSIX2_SW_DEV

;;; supported POSIX version
    _POSIX_VERSION	_POSIX2_C_VERSION

;;; sysconf constants (lexicographic order, lines then rows)
    _SC_2_CHAR_TERM		_SC_2_C_BIND		_SC_2_C_DEV
    _SC_2_C_VERSION		_SC_2_FORT_DEV		_SC_2_FORT_RUN
    _SC_2_LOCALEDEF		_SC_2_PBS		_SC_2_PBS_ACCOUNTING
    _SC_2_PBS_CHECKPOINT	_SC_2_PBS_LOCATE	_SC_2_PBS_MESSAGE
    _SC_2_PBS_TRACK		_SC_2_SW_DEV		_SC_2_UPE
    _SC_2_VERSION		_SC_ADVISORY_INFO	_SC_AIO_LISTIO_MAX
    _SC_AIO_MAX			_SC_AIO_PRIO_DELTA_MAX	_SC_ARG_MAX
    _SC_ASYNCHRONOUS_IO		_SC_ATEXIT_MAX		_SC_AVPHYS_PAGES
    _SC_BARRIERS		_SC_BASE		_SC_BC_BASE_MAX
    _SC_BC_DIM_MAX		_SC_BC_SCALE_MAX	_SC_BC_STRING_MAX
    _SC_CHARCLASS_NAME_MAX	_SC_CHAR_BIT		_SC_CHAR_MAX
    _SC_CHAR_MIN		_SC_CHILD_MAX		_SC_CLK_TCK
    _SC_CLOCK_SELECTION		_SC_COLL_WEIGHTS_MAX	_SC_CPUTIME
    _SC_C_LANG_SUPPORT		_SC_C_LANG_SUPPORT_R	_SC_DELAYTIMER_MAX
    _SC_DEVICE_IO		_SC_DEVICE_SPECIFIC	_SC_DEVICE_SPECIFIC_R
    _SC_EQUIV_CLASS_MAX		_SC_EXPR_NEST_MAX	_SC_FD_MGMT
    _SC_FIFO			_SC_FILE_ATTRIBUTES	_SC_FILE_LOCKING
    _SC_FILE_SYSTEM		_SC_FSYNC		_SC_GETGR_R_SIZE_MAX
    _SC_GETPW_R_SIZE_MAX	_SC_HOST_NAME_MAX	_SC_INT_MAX
    _SC_INT_MIN			_SC_IOV_MAX		_SC_IPV6 _SC_RAW_SOCKETS
    _SC_JOB_CONTROL		_SC_LEVEL1_DCACHE_ASSOC	_SC_LEVEL1_DCACHE_LINESIZE
    _SC_LEVEL1_DCACHE_SIZE	_SC_LEVEL1_ICACHE_ASSOC	_SC_LEVEL1_ICACHE_LINESIZE
    _SC_LEVEL1_ICACHE_SIZE	_SC_LEVEL2_CACHE_ASSOC	_SC_LEVEL2_CACHE_LINESIZE
    _SC_LEVEL2_CACHE_SIZE	_SC_LEVEL3_CACHE_ASSOC	_SC_LEVEL3_CACHE_LINESIZE
    _SC_LEVEL3_CACHE_SIZE	_SC_LEVEL4_CACHE_ASSOC	_SC_LEVEL4_CACHE_LINESIZE
    _SC_LEVEL4_CACHE_SIZE	_SC_LINE_MAX		_SC_LOGIN_NAME_MAX
    _SC_LONG_BIT		_SC_MAPPED_FILES	_SC_MB_LEN_MAX
    _SC_MEMLOCK			_SC_MEMLOCK_RANGE	_SC_MEMORY_PROTECTION
    _SC_MESSAGE_PASSING		_SC_MONOTONIC_CLOCK	_SC_MQ_OPEN_MAX
    _SC_MQ_PRIO_MAX		_SC_MULTI_PROCESS	_SC_NETWORKING
    _SC_NGROUPS_MAX		_SC_NL_ARGMAX		_SC_NL_LANGMAX
    _SC_NL_MSGMAX		_SC_NL_NMAX		_SC_NL_SETMAX
    _SC_NL_TEXTMAX		_SC_NPROCESSORS_CONF	_SC_NPROCESSORS_ONLN
    _SC_NZERO			_SC_OPEN_MAX		_SC_PAGESIZE
    _SC_PAGE_SIZE		_SC_PASS_MAX		_SC_PHYS_PAGES
    _SC_PII			_SC_PII_INTERNET	_SC_PII_INTERNET_DGRAM
    _SC_PII_INTERNET_STREAM	_SC_PII_OSI		_SC_PII_OSI_CLTS
    _SC_PII_OSI_COTS		_SC_PII_OSI_M		_SC_PII_SOCKET
    _SC_PII_XTI			_SC_PIPE		_SC_POLL
    _SC_PRIORITIZED_IO		_SC_PRIORITY_SCHEDULING	_SC_READER_WRITER_LOCKS
    _SC_REALTIME_SIGNALS	_SC_REGEXP		_SC_REGEX_VERSION
    _SC_RE_DUP_MAX		_SC_RTSIG_MAX		_SC_SAVED_IDS
    _SC_SCHAR_MAX		_SC_SCHAR_MIN		_SC_SELECT
    _SC_SEMAPHORES		_SC_SEM_NSEMS_MAX	_SC_SEM_VALUE_MAX
    _SC_SHARED_MEMORY_OBJECTS	_SC_SHELL		_SC_SHRT_MAX
    _SC_SHRT_MIN		_SC_SIGNALS		_SC_SIGQUEUE_MAX
    _SC_SINGLE_PROCESS		_SC_SPAWN		_SC_SPIN_LOCKS
    _SC_SPORADIC_SERVER		_SC_SSIZE_MAX		_SC_STREAMS
    _SC_STREAM_MAX		_SC_SYMLOOP_MAX		_SC_SYNCHRONIZED_IO
    _SC_SYSTEM_DATABASE		_SC_SYSTEM_DATABASE_R	_SC_THREADS
    _SC_THREAD_ATTR_STACKADDR	_SC_THREAD_ATTR_STACKSIZE _SC_THREAD_CPUTIME
    _SC_THREAD_DESTRUCTOR_ITERATIONS _SC_THREAD_KEYS_MAX _SC_THREAD_PRIORITY_SCHEDULING
    _SC_THREAD_PRIO_INHERIT	_SC_THREAD_PRIO_PROTECT	_SC_THREAD_PROCESS_SHARED
    _SC_THREAD_SAFE_FUNCTIONS	_SC_THREAD_SPORADIC_SERVER _SC_THREAD_STACK_MIN
    _SC_THREAD_THREADS_MAX	_SC_TIMEOUTS		_SC_TIMERS
    _SC_TIMER_MAX		_SC_TRACE		_SC_TRACE_EVENT_FILTER
    _SC_TRACE_INHERIT		_SC_TRACE_LOG		_SC_TTY_NAME_MAX
    _SC_TYPED_MEMORY_OBJECTS	_SC_TZNAME_MAX		_SC_T_IOV_MAX
    _SC_UCHAR_MAX		_SC_UINT_MAX		_SC_UIO_MAXIOV
    _SC_ULONG_MAX		_SC_USER_GROUPS		_SC_USER_GROUPS_R
    _SC_USHRT_MAX		_SC_V6_ILP32_OFF32	_SC_V6_ILP32_OFFBIG
    _SC_V6_LP64_OFF64		_SC_V6_LPBIG_OFFBIG	_SC_VERSION
    _SC_WORD_BIT		_SC_XBS5_ILP32_OFF32	_SC_XBS5_ILP32_OFFBIG
    _SC_XBS5_LP64_OFF64		_SC_XBS5_LPBIG_OFFBIG	_SC_XOPEN_CRYPT
    _SC_XOPEN_ENH_I18N		_SC_XOPEN_LEGACY	_SC_XOPEN_REALTIME
    _SC_XOPEN_REALTIME_THREADS	_SC_XOPEN_SHM		_SC_XOPEN_UNIX
    _SC_XOPEN_VERSION		_SC_XOPEN_XCU_VERSION	_SC_XOPEN_XPG2
    _SC_XOPEN_XPG3		_SC_XOPEN_XPG4

;;; limits on file system capacity
    LINK_MAX	MAX_CANON	MAX_INPUT
    NAME_MAX	PATH_MAX	PIPE_BUF
    MAXNAMLEN	FILENAME_MAX

;;; optional features in file support
    _POSIX_CHOWN_RESTRICTED
    _POSIX_NO_TRUNC
    _POSIX_VDISABLE

;;; minimum values for file system limits
    _POSIX_LINK_MAX		_POSIX_MAX_CANON
    _POSIX_MAX_INPUT		_POSIX_NAME_MAX
    _POSIX_PATH_MAX		_POSIX_PIPE_BUF
    SYMLINK_MAX			POSIX_REC_INCR_XFER_SIZE
    POSIX_REC_MAX_XFER_SIZE	POSIX_REC_MIN_XFER_SIZE
    POSIX_REC_XFER_ALIGN

;;; constants for pathconf and fpathconf
    _PC_LINK_MAX		_PC_MAX_CANON		_PC_MAX_INPUT
    _PC_NAME_MAX		_PC_PATH_MAX		_PC_PIPE_BUF
    _PC_CHOWN_RESTRICTED	_PC_NO_TRUNC		_PC_VDISABLE
    _PC_SYNC_IO			_PC_ASYNC_IO		_PC_PRIO_IO
    _PC_FILESIZEBITS		_PC_REC_INCR_XFER_SIZE	_PC_REC_MAX_XFER_SIZE
    _PC_REC_MIN_XFER_SIZE	_PC_REC_XFER_ALIGN

;;; glibc system inspection constants
    BC_BASE_MAX			_POSIX_BC_BASE_MAX
    BC_DIM_MAX			_POSIX_BC_DIM_MAX
    BC_SCALE_MAX		_POSIX_BC_SCALE_MAX
    BC_STRING_MAX		_POSIX_BC_STRING_MAX
    COLL_WEIGHTS_MAX		_POSIX_COLL_WEIGHTS_MAX
    EXPR_NEXT_MAX		_POSIX_EXPR_NEXT_MAX
    LINE_MAX			_POSIX_LINE_MAX
    EQUIV_CLASS_MAX		_POSIX_EQUIV_CLASS_MAX

;;; string-valued system configuration parameters
    _CS_PATH			_CS_LFS_CFLAGS
    _CS_LFS_LDFLAGS		_CS_LFS_LIBS
    _CS_LFS_LINTFLAGS		_CS_LFS64_CFLAGS
    _CS_LFS64_LDFLAGS		_CS_LFS64_LIBS
    _CS_LFS64_LINTFLAGS

;;; file system files pathnames
    _PATH_MNTTAB		_PATH_FSTAB
    _PATH_MOUNTED

;;; mount options
    FSTAB_RW			FSTAB_RQ
    FSTAB_RO			FSTAB_SW
    FSTAB_XX

;;; values for the mnt_type field of struct mtab
    MNTTYPE_IGNORE		MNTTYPE_NFS
    MNTTYPE_SWAP

;;; values for the mnt_opts field of struct mtab
    MNTOPT_DEFAULTS		MNTOPT_RO
    MNTOPT_RW			MNTOPT_SUID
    MNTOPT_NOSUID		MNTOPT_NOAUTO

;;; options for mount
    MS_MGC_MASK		MS_REMOUNT	MS_RDONLY
    S_IMMUTABLE		S_APPEND	MS_NOSUID
    MS_NOEXEC		MS_NODEV	MS_SYNCHRONOUS
    MS_MANDLOCK		MS_NOATIME	MS_NODIRATIME

;;; flags for umount2
    MNT_FORCE

;;; sockets constants
    PF_UNSPEC		PF_LOCAL	PF_UNIX
    PF_FILE		PF_INET		PF_AX25
    PF_IPX		PF_APPLETALK	PF_NETROM
    PF_BRIDGE		PF_ATMPVC	PF_X25
    PF_INET6		PF_ROSE		PF_DECnet
    PF_NETBEUI		PF_SECURITY	PF_KEY
    PF_NETLINK		PF_ROUTE	PF_PACKET
    PF_ASH		PF_ECONET	PF_ATMSVC
    PF_RDS		PF_SNA		PF_IRDA
    PF_PPPOX		PF_WANPIPE	PF_LLC
    PF_CAN		PF_TIPC		PF_BLUETOOTH
    PF_IUCV		PF_RXRPC	PF_ISDN
    PF_PHONET		PF_IEEE802154	PF_CAIF
    PF_ALG		PF_NFC		PF_MAX

    AF_UNSPEC		AF_LOCAL	AF_UNIX
    AF_FILE		AF_INET		AF_AX25
    AF_IPX		AF_APPLETALK	AF_NETROM
    AF_BRIDGE		AF_ATMPVC	AF_X25
    AF_INET6		AF_ROSE		AF_DECnet
    AF_NETBEUI		AF_SECURITY	AF_KEY
    AF_NETLINK		AF_ROUTE	AF_PACKET
    AF_ASH		AF_ECONET	AF_ATMSVC
    AF_RDS		AF_SNA		AF_IRDA
    AF_PPPOX		AF_WANPIPE	AF_LLC
    AF_CAN		AF_TIPC		AF_BLUETOOTH
    AF_IUCV		AF_RXRPC	AF_ISDN
    AF_PHONET		AF_IEEE802154	AF_CAIF
    AF_ALG		AF_NFC		AF_MAX

    SOCK_STREAM		SOCK_DGRAM	SOCK_RAW
    SOCK_RDM		SOCK_SEQPACKET	SOCK_DCCP
    SOCK_PACKET		SOCK_CLOEXEC	SOCK_NONBLOCK

    SHUT_RD		SHUT_WR		SHUT_RDWR
    IPPORT_RESERVED	IPPORT_USERRESERVED
    IFNAMSIZ

    MSG_OOB		MSG_PEEK	MSG_DONTROUTE
    MSG_TRYHARD		MSG_CTRUNC	MSG_PROXY
    MSG_TRUNC		MSG_DONTWAIT	MSG_EOR
    MSG_WAITALL		MSG_FIN		MSG_SYN
    MSG_CONFIRM		MSG_RST		MSG_ERRQUEUE
    MSG_NOSIGNAL	MSG_MORE	MSG_WAITFORONE
    MSG_CMSG_CLOEXEC

    SOL_SOCKET		SOL_RAW		SOL_DECNET
    SOL_X25		SOL_PACKET	SOL_ATM
    SOL_AAL		SOL_IRDA

    SO_DEBUG		SO_REUSEADDR
    SO_KEEPALIVE	SO_DONTROUTE	SO_LINGER
    SO_BROADCAST	SO_OOBINLINE	SO_SNDBUF
    SO_RCVBUF		SO_TYPE		SO_STYLE
    SO_ERROR

    SOMAXCONN

    INADDR_LOOPBACK	INADDR_ANY	INADDR_BROADCAST
    INADDR_NONE		INET_ADDRSTRLEN	INET6_ADDRSTRLEN

    AI_ADDRCONFIG	AI_ALL		AI_CANONNAME
    AI_NUMERIC_HOST	AI_NUMERIC_SERV	AI_PASSIVE
    AI_V4MAPPED		AI_NUMERICHOST
    AI_IDN		AI_CANONIDN
    AI_IDN_ALLOW_UNASSIGNED	AI_IDN_USE_STD3_ASCII_RULES

    EAI_ADDRFAMILY	EAI_AGAIN	EAI_BADFLAGS
    EAI_FAIL		EAI_FAMILY	EAI_MEMORY
    EAI_NODATA		EAI_NONAME	EAI_SERVICE
    EAI_SOCKTYPE	EAI_SYSTEM

;;; mathematics
    M_E			M_LOG2E		M_LOG10E
    M_LN2		M_LN10		M_PI
    M_PI_2		M_PI_4		M_1_PI
    M_2_PI		M_2_SQRTPI	M_SQRT2
    M_SQRT1_2

    FP_NAN		FP_INFINITE	FP_ZERO
    FP_SUBNORMAL	FP_NORMAL

;;; fnmatch()
    FNM_NOMATCH		FNM_FILE_NAME	FNM_PATHNAME
    FNM_PERIOD		FNM_NOESCAPE	FNM_LEADING_DIR
    FNM_CASEFOLD	FNM_EXTMATCH

;;;; glob() flags
    GLOB_ERR		GLOB_MARK	GLOB_NOCHECK
    GLOB_NOSORT		GLOB_NOESCAPE	GLOB_PERIOD
    GLOB_MAGCHAR	GLOB_BRACE	GLOB_NOMAGIC
    GLOB_TILDE		GLOB_TILDE_CHECK GLOB_ONLYDIR
    GLOB_ABORTED	GLOB_NOMATCH	GLOB_NOSPACE

;;;; POSIX regexps
    REG_BADBR		REG_BADPAT	REG_BADRPT
    REG_ECOLLATE	REG_ECTYPE	REG_EESCAPE
    REG_ESUBREG		REG_EBRACK	REG_EPAREN
    REG_EBRACE		REG_ERANGE

    REG_EXTENDED	REG_ICASE	REG_NOSUB
    REG_NEWLINE		REG_NOTBOL	REG_NOTEOL
    REG_NOMATCH		REG_ESPACE

;;;; word expansion
    WRDE_NOCMD		WRDE_SHOWERR	WRDE_UNDEF

    WRDE_BADCHAR	WRDE_BADVAL	WRDE_CMDSUB
    WRDE_NOSPACE	WRDE_SYNTAX

;;;; signalfd
    SFD_CLOEXEC		SFD_NONBLOCK

;;;; timerfd
    CLOCK_REALTIME	CLOCK_MONOTONIC
    TFD_CLOEXEC		TFD_NONBLOCK
    TFD_TIMER_ABSTIME

;;;; inotify
    IN_NONBLOCK		IN_CLOEXEC

    IN_ACCESS		IN_ATTRIB	IN_CLOSE_WRITE
    IN_CLOSE_NOWRITE	IN_CREATE	IN_DELETE
    IN_DELETE_SELF	IN_MODIFY	IN_MOVE_SELF
    IN_MOVED_FROM	IN_MOVED_TO	IN_OPEN
    IN_ALL_EVENTS	IN_MOVE		IN_CLOSE

    IN_DONT_FOLLOW	IN_EXCL_UNLINK	IN_MASK_ADD
    IN_ONESHOT		IN_ONLYDIR	IN_IGNORED
    IN_ISDIR		IN_Q_OVERFLOW	IN_UNMOUNT

;;;; clock
    CLOCK_MONOTONIC_RAW CLOCK_PROCESS_CPUTIME_ID
    CLOCK_THREAD_CPUTIME_ID
    TIMER_ABSTIME

;;;; struct "siginfo_t", values for field `si_code'
    SI_ASYNCNL		SI_TKILL		SI_SIGIO
    SI_ASYNCIO		SI_MESGQ		SI_TIMER
    SI_QUEUE		SI_USER			SI_KERNEL

;;;; struct "siginfo_t", `si_code' values for SIGILL signal
    ILL_ILLOPC		ILL_ILLOPN		ILL_ILLADR
    ILL_ILLTRP		ILL_PRVOPC		ILL_PRVREG
    ILL_COPROC		ILL_BADSTK

;;;; struct "siginfo_t", `si_code' values for SIGFPE signal
    FPE_INTDIV		FPE_INTOVF		FPE_FLTDIV
    FPE_FLTOVF		FPE_FLTUND		FPE_FLTRES
    FPE_FLTINV		FPE_FLTSUB

;;;; struct "siginfo_t", `si_code' values for SIGSEGV signal
    SEGV_MAPERR		SEGV_ACCERR

;;;; struct "siginfo_t", `si_code' values for SIGBUS signal
    BUS_ADRALN		BUS_ADRERR		BUS_OBJERR

;;;; struct "siginfo_t", `si_code' values for SIGTRAP signal
    TRAP_BRKPT		TRAP_TRACE

;;;; struct "siginfo_t", `si_code' values for SIGCHLD signal.
;;;
;;;Already exported above.
;;;
;;; CLD_EXITED		CLD_KILLED		CLD_DUMPED
;;; CLD_TRAPPED		CLD_STOPPED		CLD_CONTINUED

;;;; struct "siginfo_t", `si_code' values for SIGPOLL signal
    POLL_IN		POLL_OUT		POLL_MSG
    POLL_ERR		POLL_PRI		POLL_HUP

;;;; struct "sigevent_t", `sigev_notify' values.
    SIGEV_SIGNAL	SIGEV_NONE		SIGEV_THREAD
    SIGEV_THREAD_ID

;;; For "setrlimit()" and similar.
    RLIMIT_AS		RLIMIT_CORE
    RLIMIT_CPU		RLIMIT_DATA		RLIMIT_FSIZE
    RLIMIT_LOCKS	RLIMIT_MEMLOCK		RLIMIT_MSGQUEUE
    RLIMIT_NICE		RLIMIT_NOFILE		RLIMIT_NPROC
    RLIMIT_RSS		RLIMIT_RTPRIO		RLIMIT_RTTIME
    RLIMIT_SIGPENDING	RLIMIT_STACK

;;;; For "getrusage()".
    RUSAGE_SELF		RUSAGE_CHILDREN		RUSAGE_THREAD

;;;; Constants from "netinet/in.h"
    IPPROTO_IP		IPPROTO_HOPOPTS		IPPROTO_ICMP
    IPPROTO_IGMP	IPPROTO_IPIP		IPPROTO_TCP
    IPPROTO_EGP		IPPROTO_PUP		IPPROTO_UDP
    IPPROTO_IDP		IPPROTO_TP		IPPROTO_DCCP
    IPPROTO_IPV6	IPPROTO_ROUTING		IPPROTO_FRAGMENT
    IPPROTO_RSVP	IPPROTO_GRE		IPPROTO_ESP
    IPPROTO_AH		IPPROTO_ICMPV6		IPPROTO_NONE
    IPPROTO_DSTOPTS	IPPROTO_MTP		IPPROTO_ENCAP
    IPPROTO_PIM		IPPROTO_COMP		IPPROTO_SCTP
    IPPROTO_UDPLITE	IPPROTO_RAW

    IPPORT_ECHO		IPPORT_DISCARD		IPPORT_SYSTAT
    IPPORT_DAYTIME	IPPORT_NETSTAT		IPPORT_FTP
    IPPORT_TELNET	IPPORT_SMTP		IPPORT_TIMESERVER
    IPPORT_NAMESERVER	IPPORT_WHOIS		IPPORT_MTP
    IPPORT_TFTP		IPPORT_RJE		IPPORT_FINGER
    IPPORT_TTYLINK	IPPORT_SUPDUP		IPPORT_EXECSERVER
    IPPORT_LOGINSERVER	IPPORT_CMDSERVER	IPPORT_EFSSERVER
    IPPORT_BIFFUDP	IPPORT_WHOSERVER	IPPORT_ROUTESERVER
    ;;These have been already exported above.
    #;IPPORT_RESERVED	#;IPPORT_USERRESERVED

    IN_CLASSA_NET	IN_CLASSA_NSHIFT	IN_CLASSA_HOST		IN_CLASSA_MAX
    IN_CLASSB_NET	IN_CLASSB_NSHIFT	IN_CLASSB_HOST		IN_CLASSB_MAX
    IN_CLASSC_NET	IN_CLASSC_NSHIFT	IN_CLASSC_HOST

;;;; Constants from "netinet/tcp.h"
    TCP_NODELAY		TCP_MAXSEG		TCP_CORK
    TCP_KEEPIDLE	TCP_KEEPINTVL		TCP_KEEPCNT
    TCP_SYNCNT		TCP_LINGER2		TCP_DEFER_ACCEPT
    TCP_WINDOW_CLAMP	TCP_INFO		TCP_QUICKACK
    TCP_CONGESTION	TCP_MD5SIG		TCP_ESTABLISHED
    TCP_SYN_SENT	TCP_SYN_RECV		TCP_FIN_WAIT1
    TCP_FIN_WAIT2	TCP_TIME_WAIT		TCP_CLOSE
    TCP_CLOSE_WAIT	TCP_LAST_ACK		TCP_LISTEN
    TCP_CLOSING		TCPOPT_EOL		TCPOPT_NOP
    TCPOPT_MAXSEG	TCPOLEN_MAXSEG		TCPOPT_WINDOW
    TCPOLEN_WINDOW	TCPOPT_SACK_PERMITTED	TCPOLEN_SACK_PERMITTED
    TCPOPT_SACK		TCPOPT_TIMESTAMP	TCPOLEN_TIMESTAMP
    TCPOLEN_TSTAMP_APPA	TCPOPT_TSTAMP_HDR	TCP_MSS
    TCP_MAXWIN		TCP_MAX_WINSHIFT	SOL_TCP
    TCPI_OPT_TIMESTAMPS	TCPI_OPT_SACK		TCPI_OPT_WSCALE
    TCPI_OPT_ECN	TCP_CA_Open		TCP_CA_Disorder
    TCP_CA_CWR		TCP_CA_Recovery		TCP_CA_Loss
    TCP_MD5SIG_MAXKEYLEN

;;;; Constants from "netinet/udp.h"
    UDP_CORK		UDP_ENCAP		UDP_ENCAP_ESPINUDP_NON_IKE
    UDP_ENCAP_ESPINUDP	UDP_ENCAP_L2TPINUDP	SOL_UDP

;;;; Constants from "linux/icmp.h"
    ICMP_ECHOREPLY	ICMP_DEST_UNREACH	ICMP_SOURCE_QUENCH
    ICMP_REDIRECT	ICMP_ECHO		ICMP_TIME_EXCEEDED
    ICMP_PARAMETERPROB	ICMP_TIMESTAMP		ICMP_TIMESTAMPREPLY
    ICMP_INFO_REQUEST	ICMP_INFO_REPLY		ICMP_ADDRESS
    ICMP_ADDRESSREPLY	NR_ICMP_TYPES		ICMP_NET_UNREACH
    ICMP_HOST_UNREACH	ICMP_PROT_UNREACH	ICMP_PORT_UNREACH
    ICMP_FRAG_NEEDED	ICMP_SR_FAILED		ICMP_NET_UNKNOWN
    ICMP_HOST_UNKNOWN	ICMP_HOST_ISOLATED	ICMP_NET_ANO
    ICMP_HOST_ANO	ICMP_NET_UNR_TOS	ICMP_HOST_UNR_TOS
    ICMP_PKT_FILTERED	ICMP_PREC_VIOLATION	ICMP_PREC_CUTOFF
    NR_ICMP_UNREACH	ICMP_REDIR_NET		ICMP_REDIR_HOST
    ICMP_REDIR_NETTOS	ICMP_REDIR_HOSTTOS	ICMP_EXC_TTL
    ICMP_EXC_FRAGTIME	ICMP_FILTER

;;;; Constants from "netinet/igmp.h"
    IGMP_MINLEN			IGMP_MEMBERSHIP_QUERY		IGMP_V1_MEMBERSHIP_REPORT
    IGMP_V2_MEMBERSHIP_REPORT	IGMP_V2_LEAVE_GROUP		IGMP_DVMRP
    IGMP_PIM			IGMP_TRACE			IGMP_MTRACE_RESP
    IGMP_MTRACE			IGMP_MAX_HOST_REPORT_DELAY	IGMP_TIMER_SCALE
    IGMP_DELAYING_MEMBER	IGMP_IDLE_MEMBER		IGMP_LAZY_MEMBER
    IGMP_SLEEPING_MEMBER	IGMP_AWAKENING_MEMBER		IGMP_v1_ROUTER
    IGMP_v2_ROUTER

;;; --------------------------------------------------------------------

;;; errno codes, reexported from (vicare platform errno)
    E2BIG		EACCES		EADDRINUSE
    EADDRNOTAVAIL	EADV		EAFNOSUPPORT
    EAGAIN		EALREADY	EBADE
    EBADF		EBADFD		EBADMSG
    EBADR		EBADRQC		EBADSLT
    EBFONT		EBUSY		ECANCELED
    ECHILD		ECHRNG		ECOMM
    ECONNABORTED	ECONNREFUSED	ECONNRESET
    EDEADLK		EDEADLOCK	EDESTADDRREQ
    EDOM		EDOTDOT		EDQUOT
    EEXIST		EFAULT		EFBIG
    EHOSTDOWN		EHOSTUNREACH	EIDRM
    EILSEQ		EINPROGRESS	EINTR
    EINVAL		EIO		EISCONN
    EISDIR		EISNAM		EKEYEXPIRED
    EKEYREJECTED	EKEYREVOKED	EL2HLT
    EL2NSYNC		EL3HLT		EL3RST
    ELIBACC		ELIBBAD		ELIBEXEC
    ELIBMAX		ELIBSCN		ELNRNG
    ELOOP		EMEDIUMTYPE	EMFILE
    EMLINK		EMSGSIZE	EMULTIHOP
    ENAMETOOLONG	ENAVAIL		ENETDOWN
    ENETRESET		ENETUNREACH	ENFILE
    ENOANO		ENOBUFS		ENOCSI
    ENODATA		ENODEV		ENOENT
    ENOEXEC		ENOKEY		ENOLCK
    ENOLINK		ENOMEDIUM	ENOMEM
    ENOMSG		ENONET		ENOPKG
    ENOPROTOOPT		ENOSPC		ENOSR
    ENOSTR		ENOSYS		ENOTBLK
    ENOTCONN		ENOTDIR		ENOTEMPTY
    ENOTNAM		ENOTRECOVERABLE	ENOTSOCK
    ENOTTY		ENOTUNIQ	ENXIO
    EOPNOTSUPP		EOVERFLOW	EOWNERDEAD
    EPERM		EPFNOSUPPORT	EPIPE
    EPROTO		EPROTONOSUPPORT	EPROTOTYPE
    ERANGE		EREMCHG		EREMOTE
    EREMOTEIO		ERESTART	EROFS
    ESHUTDOWN		ESOCKTNOSUPPORT	ESPIPE
    ESRCH		ESRMNT		ESTALE
    ESTRPIPE		ETIME		ETIMEDOUT
    ETOOMANYREFS	ETXTBSY		EUCLEAN
    EUNATCH		EUSERS		EWOULDBLOCK
    EXDEV		EXFULL

;;; h_errno codes
    h_errno-code	case-h_errno

    ;; codes
    HOST_NOT_FOUND	TRY_AGAIN	NO_RECOVERY
    NO_ADDRESS

;;; interprocess signals
    NSIG

    ;; program error
    SIGFPE		SIGILL		SIGSEGV
    SIGBUS		SIGABRT		SIGIOT
    SIGTRAP		SIGEMT		SIGSYS

    ;; termination
    SIGTERM		SIGINT		SIGQUIT
    SIGKILL		SIGHUP

    ;; alarm
    SIGALRM		SIGVRALRM	SIGPROF

    ;; asynchronous I/O
    SIGIO		SIGURG		SIGPOLL

    ;; job control
    SIGCHLD		SIGCLD		SIGCONT
    SIGSTOP		SIGTSTP		SIGTTIN
    SIGTTOU

    ;; operation error
    SIGPIPE		SIGLOST		SIGXCPU
    SIGXSFZ

    ;; misc
    SIGUSR1		SIGUSR2		SIGWINCH
    SIGINFO)
  (import (vicare)
    (vicare platform errno)
    (only (vicare system $fx)
	  $fx=))


;;;; helpers

(define-syntax neg
  (lambda (stx)
    (syntax-case stx ()
      ((_ ?code)
       (let ((N (syntax->datum #'?code)))
	 (if (number? N)
	     (- N)
	   #t))))))


;;;; miscellaneous platform constants

(define-inline-constant SIZEOF_CHAR		1)
(define-inline-constant SIZEOF_SHORT		2)
(define-inline-constant SIZEOF_INT		4)
(define-inline-constant SIZEOF_LONG		8)
(define-inline-constant SIZEOF_LONG_LONG	8)
(define-inline-constant SIZEOF_SIZE_T		8)
(define-inline-constant SIZEOF_SSIZE_T		8)
(define-inline-constant SIZEOF_OFF_T		8)

(define-inline-constant U32MAX		4294967295)	#;(- (expt 2 32) 1)
(define-inline-constant U32MIN		0)
(define-inline-constant S32MAX		+2147483647)	#;(- (expt 2 31) 1)
(define-inline-constant S32MIN		-2147483648)	#;(- (expt 2 31))

(define-inline-constant U64MAX		18446744073709551615)	#;(- (expt 2 64) 1)
(define-inline-constant U64MIN		0)
(define-inline-constant S64MAX		+9223372036854775807)	#;(- (expt 2 63) 1)
(define-inline-constant S64MIN		-9223372036854775808)	#;(- (expt 2 63))

(define-syntax SIZE_T_MAX
  (if (= 4 8)
      (identifier-syntax U32MAX)
    (identifier-syntax U64MAX)))

(define-inline-constant SIZE_T_MIN	0)

(define-syntax SSIZE_T_MAX
  (if (= 4 8)
      (identifier-syntax S32MAX)
    (identifier-syntax S64MAX)))

(define-syntax SSIZE_T_MIN
  (if (= 4 8)
      (identifier-syntax S32MIN)
    (identifier-syntax S64MIN)))

(define-syntax OFF_T_MAX
  (if (= 4 8)
      (identifier-syntax S32MAX)
    (identifier-syntax S64MAX)))

(define-syntax OFF_T_MIN
  (if (= 4 8)
      (identifier-syntax S32MIN)
    (identifier-syntax S64MIN)))

(define-inline-constant CHAR_BIT		8)
(define-inline-constant SCHAR_MIN		-128)
(define-inline-constant SCHAR_MAX		127)
(define-inline-constant UCHAR_MAX		255)
(define-inline-constant CHAR_MIN		-128)
(define-inline-constant CHAR_MAX		127)
(define-inline-constant SHRT_MIN		-32768)
(define-inline-constant SHRT_MAX		32767)
(define-inline-constant USHRT_MAX		65535)
(define-inline-constant INT_MIN			-2147483648)
(define-inline-constant INT_MAX			2147483647)
(define-inline-constant UINT_MAX		4294967295)
(define-inline-constant LONG_MIN		-9223372036854775808)
(define-inline-constant LONG_MAX		9223372036854775807)
(define-inline-constant ULONG_MAX		18446744073709551615)
;; If the "long  long" constants are undefined, assume  that "long long"
;; is a 64-bit integer.
(define-inline-constant LONG_LONG_MIN		(or -9223372036854775808
						    ;; (- (expt 2 63))
						    -9223372036854775808))
(define-inline-constant LONG_LONG_MAX		(or 9223372036854775807
						    ;; (- (expt 2 63) 1)
						    +9223372036854775807))
(define-inline-constant ULONG_LONG_MAX		(or 18446744073709551615
						    ;; (- (expt 2 64) 1)
						    18446744073709551615))
(define-inline-constant WCHAR_MAX		2147483647)

;; constants for waitpid
(define-inline-constant WAIT_ANY		-1)
(define-inline-constant WAIT_MYPGRP		0)
(define-inline-constant WCONTINUED		8)
(define-inline-constant WEXITED			4)
(define-inline-constant WNOHANG			1)
(define-inline-constant WNOWAIT			16777216)
(define-inline-constant WSTOPPED		2)
(define-inline-constant WUNTRACED		2)
(define-inline-constant P_PID			1)
(define-inline-constant P_PGID			2)
(define-inline-constant P_ALL			0)
(define-inline-constant CLD_CONTINUED		6)
(define-inline-constant CLD_DUMPED		3)
(define-inline-constant CLD_EXITED		1)
(define-inline-constant CLD_KILLED		2)
(define-inline-constant CLD_STOPPED		5)
(define-inline-constant CLD_TRAPPED		4)

;;; These are not fixnums.
;;
;; (define-inline-constant __WCLONE		@VALUEOF___WCLONE@)
;; (define-inline-constant __WALL		@VALUEOF___WALL@)
;; (define-inline-constant __WNOTHREAD		@VALUEOF___WNOTHREAD@)

(define-inline-constant S_IFDIR			16384)
(define-inline-constant S_IFCHR			8192)
(define-inline-constant S_IFBLK			24576)
(define-inline-constant S_IFREG			32768)
(define-inline-constant S_IFLNK			40960)
(define-inline-constant S_IFSOCK		49152)
(define-inline-constant S_IFIFO			4096)

(define-inline-constant SEEK_SET		0)
(define-inline-constant SEEK_CUR		1)
(define-inline-constant SEEK_END		2)

(define-inline-constant O_ACCMODE		3)
(define-inline-constant O_APPEND		1024)
(define-inline-constant O_ASYNC			8192)
(define-inline-constant O_CREAT			64)
(define-inline-constant O_EXCL			128)
(define-inline-constant O_EXEC			#f)
(define-inline-constant O_EXLOCK		#f)
(define-inline-constant O_FSYNC			1052672)
(define-inline-constant O_IGNORE_CTTY		#f)
(define-inline-constant O_NDELAY		2048)
(define-inline-constant O_NOCTTY		256)
(define-inline-constant O_NOLINK		#f)
(define-inline-constant O_NONBLOCK		2048)
(define-inline-constant O_NOTRANS		#f)
(define-inline-constant O_RDONLY		0)
(define-inline-constant O_RDWR			2)
(define-inline-constant O_READ			#f)
(define-inline-constant O_SHLOCK		#f)
(define-inline-constant O_SYNC			1052672)
(define-inline-constant O_TRUNC			512)
(define-inline-constant O_WRITE			#f)
(define-inline-constant O_WRONLY		1)

(define-inline-constant O_NOATIME		262144)

(define-inline-constant O_CLOEXEC		524288)
(define-inline-constant O_DIRECT		16384)
(define-inline-constant O_DIRECTORY		65536)
(define-inline-constant O_LARGEFILE		0)
(define-inline-constant O_NOFOLLOW		131072)
(define-inline-constant FD_CLOEXEC		1)

(define-inline-constant EPOLL_CLOEXEC		524288)
(define-inline-constant EPOLL_CTL_ADD		1)
(define-inline-constant EPOLL_CTL_MOD		3)
(define-inline-constant EPOLL_CTL_DEL		2)
(define-inline-constant EPOLLIN			1)
(define-inline-constant EPOLLOUT		4)
(define-inline-constant EPOLLRDHUP		8192)
(define-inline-constant EPOLLPRI		2)
(define-inline-constant EPOLLERR		8)
(define-inline-constant EPOLLHUP		16)
(define-inline-constant EPOLLET			2147483648)
(define-inline-constant EPOLLONESHOT		1073741824)

(define-inline-constant F_DUPFD			0)
(define-inline-constant F_GETFD			1)
(define-inline-constant F_GETFL			3)
(define-inline-constant F_GETLK			5)
(define-inline-constant F_GETOWN		9)
(define-inline-constant F_SETFD			2)
(define-inline-constant F_SETFL			4)
(define-inline-constant F_SETLKW		7)
(define-inline-constant F_SETLK			6)
(define-inline-constant F_SETOWN		8)
(define-inline-constant F_RDLCK			0)
(define-inline-constant F_UNLCK			2)
(define-inline-constant F_WRLCK			1)

(define-inline-constant R_OK			4)
(define-inline-constant W_OK			2)
(define-inline-constant X_OK			1)
(define-inline-constant F_OK			0)

(define-inline-constant L_ctermid		9)
(define-inline-constant L_tmpnam		20)
(define-inline-constant CLOCKS_PER_SEC		1000000)

(define-inline-constant S_IRUSR			256)
(define-inline-constant S_IWUSR			128)
(define-inline-constant S_IXUSR			64)

(define-inline-constant S_IRGRP			32)
(define-inline-constant S_IWGRP			16)
(define-inline-constant S_IXGRP			8)

(define-inline-constant S_IROTH			4)
(define-inline-constant S_IWOTH			2)
(define-inline-constant S_IXOTH			1)

(define-inline-constant S_IRWXU			448)
(define-inline-constant S_IRWXG			56)
(define-inline-constant S_IRWXO			7)

(define-inline-constant S_ISUID			2048)
(define-inline-constant S_ISGID			1024)
(define-inline-constant S_ISVTX			512)

(define-inline-constant DT_BLK			6)
(define-inline-constant DT_CHR			2)
(define-inline-constant DT_DIR			4)
(define-inline-constant DT_FIFO			1)
(define-inline-constant DT_LNK			10)
(define-inline-constant DT_REG			8)
(define-inline-constant DT_SOCK			12)
(define-inline-constant DT_UNKNOWN		0)

(define-inline-constant FTW_F			0)
(define-inline-constant FTW_D			1)
(define-inline-constant FTW_NS			3)
(define-inline-constant FTW_DNR			2)
(define-inline-constant FTW_SL			4)
(define-inline-constant FTW_DP			5)
(define-inline-constant FTW_SLN			6)
(define-inline-constant FTW_PHYS		1)
(define-inline-constant FTW_MOUNT		2)
(define-inline-constant FTW_CHDIR		4)
(define-inline-constant FTW_DEPTH		8)
(define-inline-constant FTW_ACTIONRETVAL	16)
(define-inline-constant FTW_CONTINUE		0)
(define-inline-constant FTW_STOP		1)
(define-inline-constant FTW_SKIP_SUBTREE	2)
(define-inline-constant FTW_SKIP_SIBLINGS	3)

(define-inline-constant ITIMER_REAL		0)
(define-inline-constant ITIMER_VIRTUAL		1)
(define-inline-constant ITIMER_PROF		2)

(define-inline-constant EOF			-1)
(define-inline-constant MAXSYMLINKS		#f)
(define-inline-constant MAXPATHLEN		#f)

(define-inline-constant MAP_PRIVATE		2)
(define-inline-constant MAP_SHARED		1)
(define-inline-constant MAP_FIXED		16)
(define-inline-constant MAP_ANON		32)
(define-inline-constant MAP_ANONYMOUS		32)
(define-inline-constant MAP_32BITS		#f)
(define-inline-constant MAP_DENYWRITE		2048)
(define-inline-constant MAP_EXECUTABLE		4096)
(define-inline-constant MAP_FILE		0)
(define-inline-constant MAP_GROWSDOWN		256)
(define-inline-constant MAP_LOCKED		8192)
(define-inline-constant MAP_NONBLOCK		65536)
(define-inline-constant MAP_NORESERVE		16384)
(define-inline-constant MAP_POPULATE		32768)
(define-inline-constant MAP_STACK		131072)
(define-inline-constant MAP_AUTOGROW		#f)
(define-inline-constant MAP_AUTORESRV		#f)
(define-inline-constant MAP_COPY		#f)
(define-inline-constant PROT_READ		1)
(define-inline-constant PROT_WRITE		2)
(define-inline-constant PROT_EXEC		4)
(define-inline-constant PROT_NONE		0)
(define-inline-constant MS_SYNC			4)
(define-inline-constant MS_ASYNC		1)
(define-inline-constant MREMAP_MAYMOVE		1)

(define-inline-constant MADV_NORMAL		0)
(define-inline-constant MADV_RANDOM		1)
(define-inline-constant MADV_SEQUENTIAL		2)
(define-inline-constant MADV_WILLNEED		3)
(define-inline-constant MADV_DONTNEED		4)
(define-inline-constant POSIX_MADV_NORMAL	0)
(define-inline-constant POSIX_MADV_RANDOM	1)
(define-inline-constant POSIX_MADV_SEQUENTIAL	2)
(define-inline-constant POSIX_MADV_WILLNEED	3)
(define-inline-constant POSIX_MADV_DONTNEED	4)
(define-inline-constant MCL_CURRENT		1)
(define-inline-constant MCL_FUTURE		2)

(define-inline-constant FD_SETSIZE		1024)

(define-inline-constant F_LOCK			1)
(define-inline-constant F_TLOCK			2)
(define-inline-constant F_ULOCK			0)
(define-inline-constant F_TEST			3)

(define-inline-constant POLLIN			1)
(define-inline-constant POLLPRI			2)
(define-inline-constant POLLOUT			4)
(define-inline-constant POLLRDHUP		8192)
(define-inline-constant POLLERR			8)
(define-inline-constant POLLHUP			16)
(define-inline-constant POLLNVAL		32)

(define-inline-constant L_cuserid		9)

(define-inline-constant ARG_MAX			#f)
(define-inline-constant _POSIX_ARG_MAX		4096)
(define-inline-constant CHILD_MAX		#f)
(define-inline-constant _POSIX_CHILD_MAX	25)
(define-inline-constant OPEN_MAX		#f)
(define-inline-constant _POSIX_OPEN_MAX		20)
(define-inline-constant STREAM_MAX		#f)
(define-inline-constant _POSIX_STREAM_MAX	8)
(define-inline-constant TZNAME_MAX		#f)
(define-inline-constant _POSIX_TZNAME_MAX	6)
(define-inline-constant NGROUPS_MAX		65536)
(define-inline-constant SSIZE_MAX		9223372036854775807)
(define-inline-constant RE_DUP_MAX		32767)

(define-inline-constant _POSIX_JOB_CONTROL	1)
(define-inline-constant _POSIX_SAVED_IDS	1)
(define-inline-constant _POSIX2_C_DEV		200809)
(define-inline-constant _POSIX2_FORT_DEV	#f)
(define-inline-constant _POSIX2_FORT_RUN	#f)
(define-inline-constant _POSIX2_LOCALEDEF	200809)
(define-inline-constant _POSIX2_SW_DEV		200809)

(define-inline-constant _POSIX_VERSION		200809)
(define-inline-constant _POSIX2_C_VERSION	200809)

(define-inline-constant _SC_2_CHAR_TERM			95)
(define-inline-constant _SC_2_C_BIND			47)
(define-inline-constant _SC_2_C_DEV			48)
(define-inline-constant _SC_2_C_VERSION			96)
(define-inline-constant _SC_2_FORT_DEV			49)
(define-inline-constant _SC_2_FORT_RUN			50)
(define-inline-constant _SC_2_LOCALEDEF			52)
(define-inline-constant _SC_2_PBS			168)
(define-inline-constant _SC_2_PBS_ACCOUNTING		169)
(define-inline-constant _SC_2_PBS_CHECKPOINT		175)
(define-inline-constant _SC_2_PBS_LOCATE		170)
(define-inline-constant _SC_2_PBS_MESSAGE		171)
(define-inline-constant _SC_2_PBS_TRACK			172)
(define-inline-constant _SC_2_SW_DEV			51)
(define-inline-constant _SC_2_UPE			97)
(define-inline-constant _SC_2_VERSION			46)
(define-inline-constant _SC_ADVISORY_INFO		132)
(define-inline-constant _SC_AIO_LISTIO_MAX		23)
(define-inline-constant _SC_AIO_MAX			24)
(define-inline-constant _SC_AIO_PRIO_DELTA_MAX		25)
(define-inline-constant _SC_ARG_MAX			0)
(define-inline-constant _SC_ASYNCHRONOUS_IO		12)
(define-inline-constant _SC_ATEXIT_MAX			87)
(define-inline-constant _SC_AVPHYS_PAGES		86)
(define-inline-constant _SC_BARRIERS			133)
(define-inline-constant _SC_BASE			134)
(define-inline-constant _SC_BC_BASE_MAX			36)
(define-inline-constant _SC_BC_DIM_MAX			37)
(define-inline-constant _SC_BC_SCALE_MAX		38)
(define-inline-constant _SC_BC_STRING_MAX		39)
(define-inline-constant _SC_CHARCLASS_NAME_MAX		45)
(define-inline-constant _SC_CHAR_BIT			101)
(define-inline-constant _SC_CHAR_MAX			102)
(define-inline-constant _SC_CHAR_MIN			103)
(define-inline-constant _SC_CHILD_MAX			1)
(define-inline-constant _SC_CLK_TCK			2)
(define-inline-constant _SC_CLOCK_SELECTION		137)
(define-inline-constant _SC_COLL_WEIGHTS_MAX		40)
(define-inline-constant _SC_CPUTIME			138)
(define-inline-constant _SC_C_LANG_SUPPORT		135)
(define-inline-constant _SC_C_LANG_SUPPORT_R		136)
(define-inline-constant _SC_DELAYTIMER_MAX		26)
(define-inline-constant _SC_DEVICE_IO			140)
(define-inline-constant _SC_DEVICE_SPECIFIC		141)
(define-inline-constant _SC_DEVICE_SPECIFIC_R		142)
(define-inline-constant _SC_EQUIV_CLASS_MAX		41)
(define-inline-constant _SC_EXPR_NEST_MAX		42)
(define-inline-constant _SC_FD_MGMT			143)
(define-inline-constant _SC_FIFO			144)
(define-inline-constant _SC_FILE_ATTRIBUTES		146)
(define-inline-constant _SC_FILE_LOCKING		147)
(define-inline-constant _SC_FILE_SYSTEM			148)
(define-inline-constant _SC_FSYNC			15)
(define-inline-constant _SC_GETGR_R_SIZE_MAX		69)
(define-inline-constant _SC_GETPW_R_SIZE_MAX		70)
(define-inline-constant _SC_HOST_NAME_MAX		180)
(define-inline-constant _SC_INT_MAX			104)
(define-inline-constant _SC_INT_MIN			105)
(define-inline-constant _SC_IOV_MAX			60)
(define-inline-constant _SC_IPV6			235)
(define-inline-constant _SC_JOB_CONTROL			7)
(define-inline-constant _SC_LEVEL1_DCACHE_ASSOC		189)
(define-inline-constant _SC_LEVEL1_DCACHE_LINESIZE	190)
(define-inline-constant _SC_LEVEL1_DCACHE_SIZE		188)
(define-inline-constant _SC_LEVEL1_ICACHE_ASSOC		186)
(define-inline-constant _SC_LEVEL1_ICACHE_LINESIZE	187)
(define-inline-constant _SC_LEVEL1_ICACHE_SIZE		185)
(define-inline-constant _SC_LEVEL2_CACHE_ASSOC		192)
(define-inline-constant _SC_LEVEL2_CACHE_LINESIZE	193)
(define-inline-constant _SC_LEVEL2_CACHE_SIZE		191)
(define-inline-constant _SC_LEVEL3_CACHE_ASSOC		195)
(define-inline-constant _SC_LEVEL3_CACHE_LINESIZE	196)
(define-inline-constant _SC_LEVEL3_CACHE_SIZE		194)
(define-inline-constant _SC_LEVEL4_CACHE_ASSOC		198)
(define-inline-constant _SC_LEVEL4_CACHE_LINESIZE	199)
(define-inline-constant _SC_LEVEL4_CACHE_SIZE		197)
(define-inline-constant _SC_LINE_MAX			43)
(define-inline-constant _SC_LOGIN_NAME_MAX		71)
(define-inline-constant _SC_LONG_BIT			106)
(define-inline-constant _SC_MAPPED_FILES		16)
(define-inline-constant _SC_MB_LEN_MAX			108)
(define-inline-constant _SC_MEMLOCK			17)
(define-inline-constant _SC_MEMLOCK_RANGE		18)
(define-inline-constant _SC_MEMORY_PROTECTION		19)
(define-inline-constant _SC_MESSAGE_PASSING		20)
(define-inline-constant _SC_MONOTONIC_CLOCK		149)
(define-inline-constant _SC_MQ_OPEN_MAX			27)
(define-inline-constant _SC_MQ_PRIO_MAX			28)
(define-inline-constant _SC_MULTI_PROCESS		150)
(define-inline-constant _SC_NETWORKING			152)
(define-inline-constant _SC_NGROUPS_MAX			3)
(define-inline-constant _SC_NL_ARGMAX			119)
(define-inline-constant _SC_NL_LANGMAX			120)
(define-inline-constant _SC_NL_MSGMAX			121)
(define-inline-constant _SC_NL_NMAX			122)
(define-inline-constant _SC_NL_SETMAX			123)
(define-inline-constant _SC_NL_TEXTMAX			124)
(define-inline-constant _SC_NPROCESSORS_CONF		83)
(define-inline-constant _SC_NPROCESSORS_ONLN		84)
(define-inline-constant _SC_NZERO			109)
(define-inline-constant _SC_OPEN_MAX			4)
(define-inline-constant _SC_PAGESIZE			30)
(define-inline-constant _SC_PAGE_SIZE			30)
(define-inline-constant _SC_PASS_MAX			88)
(define-inline-constant _SC_PHYS_PAGES			85)
(define-inline-constant _SC_PII				53)
(define-inline-constant _SC_PII_INTERNET		56)
(define-inline-constant _SC_PII_INTERNET_DGRAM		62)
(define-inline-constant _SC_PII_INTERNET_STREAM		61)
(define-inline-constant _SC_PII_OSI			57)
(define-inline-constant _SC_PII_OSI_CLTS		64)
(define-inline-constant _SC_PII_OSI_COTS		63)
(define-inline-constant _SC_PII_OSI_M			65)
(define-inline-constant _SC_PII_SOCKET			55)
(define-inline-constant _SC_PII_XTI			54)
(define-inline-constant _SC_PIPE			145)
(define-inline-constant _SC_POLL			58)
(define-inline-constant _SC_PRIORITIZED_IO		13)
(define-inline-constant _SC_PRIORITY_SCHEDULING		10)
(define-inline-constant _SC_RAW_SOCKETS			236)
(define-inline-constant _SC_READER_WRITER_LOCKS		153)
(define-inline-constant _SC_REALTIME_SIGNALS		9)
(define-inline-constant _SC_REGEXP			155)
(define-inline-constant _SC_REGEX_VERSION		156)
(define-inline-constant _SC_RE_DUP_MAX			44)
(define-inline-constant _SC_RTSIG_MAX			31)
(define-inline-constant _SC_SAVED_IDS			8)
(define-inline-constant _SC_SCHAR_MAX			111)
(define-inline-constant _SC_SCHAR_MIN			112)
(define-inline-constant _SC_SELECT			59)
(define-inline-constant _SC_SEMAPHORES			21)
(define-inline-constant _SC_SEM_NSEMS_MAX		32)
(define-inline-constant _SC_SEM_VALUE_MAX		33)
(define-inline-constant _SC_SHARED_MEMORY_OBJECTS	22)
(define-inline-constant _SC_SHELL			157)
(define-inline-constant _SC_SHRT_MAX			113)
(define-inline-constant _SC_SHRT_MIN			114)
(define-inline-constant _SC_SIGNALS			158)
(define-inline-constant _SC_SIGQUEUE_MAX		34)
(define-inline-constant _SC_SINGLE_PROCESS		151)
(define-inline-constant _SC_SPAWN			159)
(define-inline-constant _SC_SPIN_LOCKS			154)
(define-inline-constant _SC_SPORADIC_SERVER		160)
(define-inline-constant _SC_SSIZE_MAX			110)
(define-inline-constant _SC_STREAMS			174)
(define-inline-constant _SC_STREAM_MAX			5)
(define-inline-constant _SC_SYMLOOP_MAX			173)
(define-inline-constant _SC_SYNCHRONIZED_IO		14)
(define-inline-constant _SC_SYSTEM_DATABASE		162)
(define-inline-constant _SC_SYSTEM_DATABASE_R		163)
(define-inline-constant _SC_THREADS			67)
(define-inline-constant _SC_THREAD_ATTR_STACKADDR	77)
(define-inline-constant _SC_THREAD_ATTR_STACKSIZE	78)
(define-inline-constant _SC_THREAD_CPUTIME		139)
(define-inline-constant _SC_THREAD_DESTRUCTOR_ITERATIONS 73)
(define-inline-constant _SC_THREAD_KEYS_MAX		74)
(define-inline-constant _SC_THREAD_PRIORITY_SCHEDULING	79)
(define-inline-constant _SC_THREAD_PRIO_INHERIT		80)
(define-inline-constant _SC_THREAD_PRIO_PROTECT		81)
(define-inline-constant _SC_THREAD_PROCESS_SHARED	82)
(define-inline-constant _SC_THREAD_SAFE_FUNCTIONS	68)
(define-inline-constant _SC_THREAD_SPORADIC_SERVER	161)
(define-inline-constant _SC_THREAD_STACK_MIN		75)
(define-inline-constant _SC_THREAD_THREADS_MAX		76)
(define-inline-constant _SC_TIMEOUTS			164)
(define-inline-constant _SC_TIMERS			11)
(define-inline-constant _SC_TIMER_MAX			35)
(define-inline-constant _SC_TRACE			181)
(define-inline-constant _SC_TRACE_EVENT_FILTER		182)
(define-inline-constant _SC_TRACE_INHERIT		183)
(define-inline-constant _SC_TRACE_LOG			184)
(define-inline-constant _SC_TTY_NAME_MAX		72)
(define-inline-constant _SC_TYPED_MEMORY_OBJECTS	165)
(define-inline-constant _SC_TZNAME_MAX			6)
(define-inline-constant _SC_T_IOV_MAX			66)
(define-inline-constant _SC_UCHAR_MAX			115)
(define-inline-constant _SC_UINT_MAX			116)
(define-inline-constant _SC_UIO_MAXIOV			60)
(define-inline-constant _SC_ULONG_MAX			117)
(define-inline-constant _SC_USER_GROUPS			166)
(define-inline-constant _SC_USER_GROUPS_R		167)
(define-inline-constant _SC_USHRT_MAX			118)
(define-inline-constant _SC_V6_ILP32_OFF32		176)
(define-inline-constant _SC_V6_ILP32_OFFBIG		177)
(define-inline-constant _SC_V6_LP64_OFF64		178)
(define-inline-constant _SC_V6_LPBIG_OFFBIG		179)
(define-inline-constant _SC_VERSION			29)
(define-inline-constant _SC_WORD_BIT			107)
(define-inline-constant _SC_XBS5_ILP32_OFF32		125)
(define-inline-constant _SC_XBS5_ILP32_OFFBIG		126)
(define-inline-constant _SC_XBS5_LP64_OFF64		127)
(define-inline-constant _SC_XBS5_LPBIG_OFFBIG		128)
(define-inline-constant _SC_XOPEN_CRYPT			92)
(define-inline-constant _SC_XOPEN_ENH_I18N		93)
(define-inline-constant _SC_XOPEN_LEGACY		129)
(define-inline-constant _SC_XOPEN_REALTIME		130)
(define-inline-constant _SC_XOPEN_REALTIME_THREADS	131)
(define-inline-constant _SC_XOPEN_SHM			94)
(define-inline-constant _SC_XOPEN_UNIX			91)
(define-inline-constant _SC_XOPEN_VERSION		89)
(define-inline-constant _SC_XOPEN_XCU_VERSION		90)
(define-inline-constant _SC_XOPEN_XPG2			98)
(define-inline-constant _SC_XOPEN_XPG3			99)
(define-inline-constant _SC_XOPEN_XPG4			100)

(define-inline-constant LINK_MAX			#f)
(define-inline-constant MAX_CANON			255)
(define-inline-constant MAX_INPUT			255)
(define-inline-constant NAME_MAX			255)
(define-inline-constant PATH_MAX			4096)
(define-inline-constant PIPE_BUF			4096)
(define-inline-constant MAXNAMLEN			255)
(define-inline-constant FILENAME_MAX			4096)

(define-inline-constant _POSIX_CHOWN_RESTRICTED		0)
(define-inline-constant _POSIX_NO_TRUNC			1)
(define-inline-constant _POSIX_VDISABLE			0)

(define-inline-constant _POSIX_LINK_MAX			8)
(define-inline-constant _POSIX_MAX_CANON		255)
(define-inline-constant _POSIX_MAX_INPUT		255)
(define-inline-constant _POSIX_NAME_MAX			14)
(define-inline-constant _POSIX_PATH_MAX			256)
(define-inline-constant _POSIX_PIPE_BUF			512)
(define-inline-constant SYMLINK_MAX			#f)
(define-inline-constant POSIX_REC_INCR_XFER_SIZE	#f)
(define-inline-constant POSIX_REC_MAX_XFER_SIZE		#f)
(define-inline-constant POSIX_REC_MIN_XFER_SIZE		#f)
(define-inline-constant POSIX_REC_XFER_ALIGN		#f)

(define-inline-constant _PC_LINK_MAX			0)
(define-inline-constant _PC_MAX_CANON			1)
(define-inline-constant _PC_MAX_INPUT			2)
(define-inline-constant _PC_NAME_MAX			3)
(define-inline-constant _PC_PATH_MAX			4)
(define-inline-constant _PC_PIPE_BUF			5)
(define-inline-constant _PC_CHOWN_RESTRICTED		6)
(define-inline-constant _PC_NO_TRUNC			7)
(define-inline-constant _PC_VDISABLE			8)
(define-inline-constant _PC_SYNC_IO			9)
(define-inline-constant _PC_ASYNC_IO			10)
(define-inline-constant _PC_PRIO_IO			11)
(define-inline-constant _PC_FILESIZEBITS		13)
(define-inline-constant _PC_REC_INCR_XFER_SIZE		14)
(define-inline-constant _PC_REC_MAX_XFER_SIZE		15)
(define-inline-constant _PC_REC_MIN_XFER_SIZE		16)
(define-inline-constant _PC_REC_XFER_ALIGN		17)

(define-inline-constant BC_BASE_MAX			99)
(define-inline-constant _POSIX_BC_BASE_MAX		#f)
(define-inline-constant BC_DIM_MAX			2048)
(define-inline-constant _POSIX_BC_DIM_MAX		#f)
(define-inline-constant BC_SCALE_MAX			99)
(define-inline-constant _POSIX_BC_SCALE_MAX		#f)
(define-inline-constant BC_STRING_MAX			1000)
(define-inline-constant _POSIX_BC_STRING_MAX		#f)
(define-inline-constant COLL_WEIGHTS_MAX		255)
(define-inline-constant _POSIX_COLL_WEIGHTS_MAX		#f)
(define-inline-constant EXPR_NEXT_MAX			#f)
(define-inline-constant _POSIX_EXPR_NEXT_MAX		#f)
(define-inline-constant LINE_MAX			2048)
(define-inline-constant _POSIX_LINE_MAX			#f)
(define-inline-constant EQUIV_CLASS_MAX			#f)
(define-inline-constant _POSIX_EQUIV_CLASS_MAX		#f)

(define-inline-constant _CS_PATH			0)
(define-inline-constant _CS_LFS_CFLAGS			1000)
(define-inline-constant _CS_LFS_LDFLAGS			1001)
(define-inline-constant _CS_LFS_LIBS			1002)
(define-inline-constant _CS_LFS_LINTFLAGS		1003)
(define-inline-constant _CS_LFS64_CFLAGS		1004)
(define-inline-constant _CS_LFS64_LDFLAGS		1005)
(define-inline-constant _CS_LFS64_LIBS			1006)
(define-inline-constant _CS_LFS64_LINTFLAGS		1007)

(define-inline-constant _PATH_MNTTAB		"/etc/fstab")
(define-inline-constant _PATH_FSTAB		"/etc/fstab")
(define-inline-constant _PATH_MOUNTED		"/etc/mtab")

(define-inline-constant FSTAB_RW		"rw")
(define-inline-constant FSTAB_RQ		"rq")
(define-inline-constant FSTAB_RO		"ro")
(define-inline-constant FSTAB_SW		"sw")
(define-inline-constant FSTAB_XX		"xx")

(define-inline-constant MNTTYPE_IGNORE		"ignore")
(define-inline-constant MNTTYPE_NFS		"nfs")
(define-inline-constant MNTTYPE_SWAP		"swap")

(define-inline-constant MNTOPT_DEFAULTS		"defaults")
(define-inline-constant MNTOPT_RO		"ro")
(define-inline-constant MNTOPT_RW		"rw")
(define-inline-constant MNTOPT_SUID		"suid")
(define-inline-constant MNTOPT_NOSUID		"nosuid")
(define-inline-constant MNTOPT_NOAUTO		"noauto")

(define-inline-constant MS_MGC_MASK		#f)
(define-inline-constant MS_REMOUNT		32)
(define-inline-constant MS_RDONLY		1)
(define-inline-constant S_IMMUTABLE		#f)
(define-inline-constant S_APPEND		#f)
(define-inline-constant MS_NOSUID		2)
(define-inline-constant MS_NOEXEC		8)
(define-inline-constant MS_NODEV		4)
(define-inline-constant MS_SYNCHRONOUS		16)
(define-inline-constant MS_MANDLOCK		64)
(define-inline-constant MS_NOATIME		1024)
(define-inline-constant MS_NODIRATIME		2048)

(define-inline-constant MNT_FORCE		1)

;; on the GNU C Library see bits/socket.h
(define-inline-constant SOCK_STREAM		1)
(define-inline-constant SOCK_DGRAM		2)
(define-inline-constant SOCK_RAW		3)
(define-inline-constant SOCK_RDM		4)
(define-inline-constant SOCK_SEQPACKET		5)
(define-inline-constant SOCK_DCCP		6)
(define-inline-constant SOCK_PACKET		10)
(define-inline-constant SOCK_CLOEXEC		524288)
(define-inline-constant SOCK_NONBLOCK		2048)

;; on the GNU C Library see bits/socket.h
(define-inline-constant PF_UNSPEC		0)
(define-inline-constant PF_LOCAL		1)
(define-inline-constant PF_UNIX			1)
(define-inline-constant PF_FILE			1)
(define-inline-constant PF_INET			2)
(define-inline-constant PF_AX25			3)
(define-inline-constant PF_IPX			4)
(define-inline-constant PF_APPLETALK		5)
(define-inline-constant PF_NETROM		6)
(define-inline-constant PF_BRIDGE		7)
(define-inline-constant PF_ATMPVC		8)
(define-inline-constant PF_X25			9)
(define-inline-constant PF_INET6		10)
(define-inline-constant PF_ROSE			11)
(define-inline-constant PF_DECnet		12)
(define-inline-constant PF_NETBEUI		13)
(define-inline-constant PF_SECURITY		14)
(define-inline-constant PF_KEY			15)
(define-inline-constant PF_NETLINK		16)
(define-inline-constant PF_ROUTE		16)
(define-inline-constant PF_PACKET		17)
(define-inline-constant PF_ASH			18)
(define-inline-constant PF_ECONET		19)
(define-inline-constant PF_ATMSVC		20)
(define-inline-constant PF_RDS			21)
(define-inline-constant PF_SNA			22)
(define-inline-constant PF_IRDA			23)
(define-inline-constant PF_PPPOX		24)
(define-inline-constant PF_WANPIPE		25)
(define-inline-constant PF_LLC			26)
(define-inline-constant PF_CAN			29)
(define-inline-constant PF_TIPC			30)
(define-inline-constant PF_BLUETOOTH		31)
(define-inline-constant PF_IUCV			32)
(define-inline-constant PF_RXRPC		33)
(define-inline-constant PF_ISDN			34)
(define-inline-constant PF_PHONET		35)
(define-inline-constant PF_IEEE802154		36)
(define-inline-constant PF_CAIF			37)
(define-inline-constant PF_ALG			38)
(define-inline-constant PF_NFC			39)
(define-inline-constant PF_MAX			41)

(define-inline-constant AF_UNSPEC		0)
(define-inline-constant AF_LOCAL		1)
(define-inline-constant AF_UNIX			1)
(define-inline-constant AF_FILE			1)
(define-inline-constant AF_INET			2)
(define-inline-constant AF_AX25			3)
(define-inline-constant AF_IPX			4)
(define-inline-constant AF_APPLETALK		5)
(define-inline-constant AF_NETROM		6)
(define-inline-constant AF_BRIDGE		7)
(define-inline-constant AF_ATMPVC		8)
(define-inline-constant AF_X25			9)
(define-inline-constant AF_INET6		10)
(define-inline-constant AF_ROSE			11)
(define-inline-constant AF_DECnet		12)
(define-inline-constant AF_NETBEUI		13)
(define-inline-constant AF_SECURITY		14)
(define-inline-constant AF_KEY			15)
(define-inline-constant AF_NETLINK		16)
(define-inline-constant AF_ROUTE		16)
(define-inline-constant AF_PACKET		17)
(define-inline-constant AF_ASH			18)
(define-inline-constant AF_ECONET		19)
(define-inline-constant AF_ATMSVC		20)
(define-inline-constant AF_RDS			21)
(define-inline-constant AF_SNA			22)
(define-inline-constant AF_IRDA			23)
(define-inline-constant AF_PPPOX		24)
(define-inline-constant AF_WANPIPE		25)
(define-inline-constant AF_LLC			26)
(define-inline-constant AF_CAN			29)
(define-inline-constant AF_TIPC			30)
(define-inline-constant AF_BLUETOOTH		31)
(define-inline-constant AF_IUCV			32)
(define-inline-constant AF_RXRPC		33)
(define-inline-constant AF_ISDN			34)
(define-inline-constant AF_PHONET		35)
(define-inline-constant AF_IEEE802154		36)
(define-inline-constant AF_CAIF			37)
(define-inline-constant AF_ALG			38)
(define-inline-constant AF_NFC			39)
(define-inline-constant AF_MAX			41)

(define-inline-constant SHUT_RD			0)
(define-inline-constant SHUT_WR			1)
(define-inline-constant SHUT_RDWR		2)
(define-inline-constant IPPORT_RESERVED		1024)
(define-inline-constant IPPORT_USERRESERVED	5000)
(define-inline-constant IFNAMSIZ		16)

;; on the GNU C Library see bits/socket.h
(define-inline-constant MSG_OOB			1)
(define-inline-constant MSG_PEEK		2)
(define-inline-constant MSG_DONTROUTE		4)
(define-inline-constant MSG_TRYHARD		4)
(define-inline-constant MSG_CTRUNC		8)
(define-inline-constant MSG_PROXY		16)
(define-inline-constant MSG_TRUNC		32)
(define-inline-constant MSG_DONTWAIT		64)
(define-inline-constant MSG_EOR			128)
(define-inline-constant MSG_WAITALL		256)
(define-inline-constant MSG_FIN			512)
(define-inline-constant MSG_SYN			1024)
(define-inline-constant MSG_CONFIRM		2048)
(define-inline-constant MSG_RST			4096)
(define-inline-constant MSG_ERRQUEUE		8192)
(define-inline-constant MSG_NOSIGNAL		16384)
(define-inline-constant MSG_MORE		32768)
(define-inline-constant MSG_WAITFORONE		65536)
(define-inline-constant MSG_CMSG_CLOEXEC	1073741824)

;; on the GNU C Library see bits/socket.h
(define-inline-constant SOL_SOCKET		1)
(define-inline-constant SOL_RAW			255)
(define-inline-constant SOL_DECNET		261)
(define-inline-constant SOL_X25			262)
(define-inline-constant SOL_PACKET		263)
(define-inline-constant SOL_ATM			264)
(define-inline-constant SOL_AAL			265)
(define-inline-constant SOL_IRDA		266)

(define-inline-constant SO_DEBUG		1)
(define-inline-constant SO_REUSEADDR		2)
(define-inline-constant SO_KEEPALIVE		9)
(define-inline-constant SO_DONTROUTE		5)
(define-inline-constant SO_LINGER		13)
(define-inline-constant SO_BROADCAST		6)
(define-inline-constant SO_OOBINLINE		10)
(define-inline-constant SO_SNDBUF		7)
(define-inline-constant SO_RCVBUF		8)
(define-inline-constant SO_TYPE			3)
(define-inline-constant SO_STYLE		#f)
(define-inline-constant SO_ERROR		4)

(define-inline-constant SOMAXCONN		128)

(define-inline-constant INADDR_LOOPBACK		2130706433)
(define-inline-constant INADDR_ANY		0)
(define-inline-constant INADDR_BROADCAST	4294967295)
(define-inline-constant INADDR_NONE		4294967295)
(define-inline-constant INET_ADDRSTRLEN		16)
(define-inline-constant INET6_ADDRSTRLEN	46)

(define-inline-constant AI_ADDRCONFIG		32)
(define-inline-constant AI_ALL			16)
(define-inline-constant AI_CANONNAME		2)
(define-inline-constant AI_NUMERIC_HOST		#f)
(define-inline-constant AI_NUMERIC_SERV		#f)
(define-inline-constant AI_PASSIVE		1)
(define-inline-constant AI_V4MAPPED		8)
(define-inline-constant AI_NUMERICHOST		4)
(define-inline-constant AI_IDN			64)
(define-inline-constant AI_CANONIDN		128)
(define-inline-constant AI_IDN_ALLOW_UNASSIGNED	256)
(define-inline-constant AI_IDN_USE_STD3_ASCII_RULES 512)

(define-inline-constant EAI_ADDRFAMILY		-9)
(define-inline-constant EAI_AGAIN		-3)
(define-inline-constant EAI_BADFLAGS		-1)
(define-inline-constant EAI_FAIL		-4)
(define-inline-constant EAI_FAMILY		-6)
(define-inline-constant EAI_MEMORY		-10)
(define-inline-constant EAI_NODATA		-5)
(define-inline-constant EAI_NONAME		-2)
(define-inline-constant EAI_SERVICE		-8)
(define-inline-constant EAI_SOCKTYPE		-7)
(define-inline-constant EAI_SYSTEM		-11)

(define-inline-constant RAND_MAX		2147483647)

;;; mathematics
(define-inline-constant M_E			2.718282)
(define-inline-constant M_LOG2E			1.442695)
(define-inline-constant M_LOG10E		0.434294)
(define-inline-constant M_LN2			0.693147)
(define-inline-constant M_LN10			2.302585)
(define-inline-constant M_PI			3.141593)
(define-inline-constant M_PI_2			1.570796)
(define-inline-constant M_PI_4			0.785398)
(define-inline-constant M_1_PI			0.318310)
(define-inline-constant M_2_PI			0.636620)
(define-inline-constant M_2_SQRTPI		1.128379)
(define-inline-constant M_SQRT2			1.414214)
(define-inline-constant M_SQRT1_2		0.707107)

(define-inline-constant FP_NAN			0)
(define-inline-constant FP_INFINITE		1)
(define-inline-constant FP_ZERO			2)
(define-inline-constant FP_SUBNORMAL		3)
(define-inline-constant FP_NORMAL		4)

;;fnmatch
(define-inline-constant FNM_NOMATCH		1)
(define-inline-constant FNM_FILE_NAME		1)
(define-inline-constant FNM_PATHNAME		1)
(define-inline-constant FNM_PERIOD		4)
(define-inline-constant FNM_NOESCAPE		2)
(define-inline-constant FNM_LEADING_DIR		8)
(define-inline-constant FNM_CASEFOLD		16)
(define-inline-constant FNM_EXTMATCH		32)

;; glob
(define-inline-constant GLOB_ERR		1)
(define-inline-constant GLOB_MARK		2)
(define-inline-constant GLOB_NOCHECK		16)
(define-inline-constant GLOB_NOSORT		4)
(define-inline-constant GLOB_NOESCAPE		64)
(define-inline-constant GLOB_PERIOD		128)
(define-inline-constant GLOB_MAGCHAR		256)
(define-inline-constant GLOB_BRACE		1024)
(define-inline-constant GLOB_NOMAGIC		2048)
(define-inline-constant GLOB_TILDE		4096)
(define-inline-constant GLOB_TILDE_CHECK	16384)
(define-inline-constant GLOB_ONLYDIR		8192)
(define-inline-constant GLOB_ABORTED		2)
(define-inline-constant GLOB_NOMATCH		3)
(define-inline-constant GLOB_NOSPACE		1)

;;;; POSIX regexps
(define-inline-constant REG_BADBR		10)
(define-inline-constant REG_BADPAT		2)
(define-inline-constant REG_BADRPT		13)
(define-inline-constant REG_ECOLLATE		3)
(define-inline-constant REG_ECTYPE		4)
(define-inline-constant REG_EESCAPE		5)
(define-inline-constant REG_ESUBREG		6)
(define-inline-constant REG_EBRACK		7)
(define-inline-constant REG_EPAREN		8)
(define-inline-constant REG_EBRACE		9)
(define-inline-constant REG_ERANGE		11)

(define-inline-constant REG_EXTENDED		1)
(define-inline-constant REG_ICASE		2)
(define-inline-constant REG_NOSUB		8)
(define-inline-constant REG_NEWLINE		4)
(define-inline-constant REG_NOTBOL		1)
(define-inline-constant REG_NOTEOL		2)
(define-inline-constant REG_NOMATCH		1)
(define-inline-constant REG_ESPACE		12)

;;;; word expansion
(define-inline-constant WRDE_NOCMD		4)
(define-inline-constant WRDE_SHOWERR		16)
(define-inline-constant WRDE_UNDEF		32)

(define-inline-constant WRDE_BADCHAR		2)
(define-inline-constant WRDE_BADVAL		3)
(define-inline-constant WRDE_CMDSUB		4)
(define-inline-constant WRDE_NOSPACE		1)
(define-inline-constant WRDE_SYNTAX		5)

;;; signalfd

(define-inline-constant SFD_CLOEXEC		524288)
(define-inline-constant SFD_NONBLOCK		2048)

;;; timerfd

(define-inline-constant CLOCK_REALTIME		0)
(define-inline-constant CLOCK_MONOTONIC		1)
(define-inline-constant TFD_CLOEXEC		524288)
(define-inline-constant TFD_NONBLOCK		2048)
(define-inline-constant TFD_TIMER_ABSTIME	1)

;;; inotify
(define-inline-constant IN_NONBLOCK		2048)
(define-inline-constant IN_CLOEXEC		524288)

(define-inline-constant IN_ACCESS		1)
(define-inline-constant IN_ATTRIB		4)
(define-inline-constant IN_CLOSE_WRITE		8)
(define-inline-constant IN_CLOSE_NOWRITE	16)
(define-inline-constant IN_CREATE		256)
(define-inline-constant IN_DELETE		512)
(define-inline-constant IN_DELETE_SELF		1024)
(define-inline-constant IN_MODIFY		2)
(define-inline-constant IN_MOVE_SELF		2048)
(define-inline-constant IN_MOVED_FROM		64)
(define-inline-constant IN_MOVED_TO		128)
(define-inline-constant IN_OPEN			32)
(define-inline-constant IN_ALL_EVENTS		4095)
(define-inline-constant IN_MOVE			192)
(define-inline-constant IN_CLOSE		24)

(define-inline-constant IN_DONT_FOLLOW		33554432)
(define-inline-constant IN_EXCL_UNLINK		67108864)
(define-inline-constant IN_MASK_ADD		536870912)
(define-inline-constant IN_ONESHOT		2147483648)
(define-inline-constant IN_ONLYDIR		16777216)
(define-inline-constant IN_IGNORED		32768)
(define-inline-constant IN_ISDIR		1073741824)
(define-inline-constant IN_Q_OVERFLOW		16384)
(define-inline-constant IN_UNMOUNT		8192)

;;; clock
(define-inline-constant CLOCK_MONOTONIC_RAW	4)
(define-inline-constant CLOCK_PROCESS_CPUTIME_ID 2)
(define-inline-constant CLOCK_THREAD_CPUTIME_ID	3)

(define-inline-constant TIMER_ABSTIME		1)

;;; --------------------------------------------------------------------

;;; struct "siginfo_t", values for field `si_code'
(define-inline-constant SI_ASYNCNL		-60)
(define-inline-constant SI_TKILL		-6)
(define-inline-constant SI_SIGIO		-5)
(define-inline-constant SI_ASYNCIO		-4)
(define-inline-constant SI_MESGQ		-3)
(define-inline-constant SI_TIMER		-2)
(define-inline-constant SI_QUEUE		-1)
(define-inline-constant SI_USER			0)
(define-inline-constant SI_KERNEL		128)

;;; struct "siginfo_t", `si_code' values for SIGILL signal
(define-inline-constant ILL_ILLOPC		1)
(define-inline-constant ILL_ILLOPN		2)
(define-inline-constant ILL_ILLADR		3)
(define-inline-constant ILL_ILLTRP		4)
(define-inline-constant ILL_PRVOPC		5)
(define-inline-constant ILL_PRVREG		6)
(define-inline-constant ILL_COPROC		7)
(define-inline-constant ILL_BADSTK		8)

;;; struct "siginfo_t", `si_code' values for SIGFPE signal
(define-inline-constant FPE_INTDIV		1)
(define-inline-constant FPE_INTOVF		2)
(define-inline-constant FPE_FLTDIV		3)
(define-inline-constant FPE_FLTOVF		4)
(define-inline-constant FPE_FLTUND		5)
(define-inline-constant FPE_FLTRES		6)
(define-inline-constant FPE_FLTINV		7)
(define-inline-constant FPE_FLTSUB		8)

;;; struct "siginfo_t", `si_code' values for SIGSEGV signal
(define-inline-constant SEGV_MAPERR		1)
(define-inline-constant SEGV_ACCERR		2)

;;; struct "siginfo_t", `si_code' values for SIGBUS signal
(define-inline-constant BUS_ADRALN		1)
(define-inline-constant BUS_ADRERR		2)
(define-inline-constant BUS_OBJERR		3)

;;; struct "siginfo_t", `si_code' values for SIGTRAP signal
(define-inline-constant TRAP_BRKPT		1)
(define-inline-constant TRAP_TRACE		2)

;;; struct "siginfo_t", `si_code' values for SIGCHLD signal.
;;
;;Already defined above.
;;
;; (define-inline-constant CLD_EXITED		1)
;; (define-inline-constant CLD_KILLED		2)
;; (define-inline-constant CLD_DUMPED		3)
;; (define-inline-constant CLD_TRAPPED		4)
;; (define-inline-constant CLD_STOPPED		5)
;; (define-inline-constant CLD_CONTINUED	6)

;;; struct "siginfo_t", `si_code' values for SIGPOLL signal
(define-inline-constant POLL_IN			1)
(define-inline-constant POLL_OUT		2)
(define-inline-constant POLL_MSG		3)
(define-inline-constant POLL_ERR		4)
(define-inline-constant POLL_PRI		5)
(define-inline-constant POLL_HUP		6)

;;; struct "sigevent_t", `sigev_notify' values.
(define-inline-constant SIGEV_SIGNAL		0)
(define-inline-constant SIGEV_NONE		1)
(define-inline-constant SIGEV_THREAD		2)
(define-inline-constant SIGEV_THREAD_ID		4)

;;; For "setrlimit()" and similar.
(define-inline-constant RLIMIT_AS		#f)
(define-inline-constant RLIMIT_CORE		#f)
(define-inline-constant RLIMIT_CPU		#f)
(define-inline-constant RLIMIT_DATA		#f)
(define-inline-constant RLIMIT_FSIZE		#f)
(define-inline-constant RLIMIT_LOCKS		#f)
(define-inline-constant RLIMIT_MEMLOCK		#f)
(define-inline-constant RLIMIT_MSGQUEUE		#f)
(define-inline-constant RLIMIT_NICE		#f)
(define-inline-constant RLIMIT_NOFILE		#f)
(define-inline-constant RLIMIT_NPROC		#f)
(define-inline-constant RLIMIT_RSS		#f)
(define-inline-constant RLIMIT_RTPRIO		#f)
(define-inline-constant RLIMIT_RTTIME		#f)
(define-inline-constant RLIMIT_SIGPENDING	#f)
(define-inline-constant RLIMIT_STACK		#f)

;;; For "getrusage()".
(define-inline-constant RUSAGE_SELF		#f)
(define-inline-constant RUSAGE_CHILDREN		#f)
(define-inline-constant RUSAGE_THREAD		#f)

;;; Constants from "netinet/in.h"
(define-inline-constant IPPROTO_IP		0)
(define-inline-constant IPPROTO_HOPOPTS		0)
(define-inline-constant IPPROTO_ICMP		1)
(define-inline-constant IPPROTO_IGMP		2)
(define-inline-constant IPPROTO_IPIP		4)
(define-inline-constant IPPROTO_TCP		6)
(define-inline-constant IPPROTO_EGP		8)
(define-inline-constant IPPROTO_PUP		12)
(define-inline-constant IPPROTO_UDP		17)
(define-inline-constant IPPROTO_IDP		22)
(define-inline-constant IPPROTO_TP		29)
(define-inline-constant IPPROTO_DCCP		33)
(define-inline-constant IPPROTO_IPV6		41)
(define-inline-constant IPPROTO_ROUTING		43)
(define-inline-constant IPPROTO_FRAGMENT	44)
(define-inline-constant IPPROTO_RSVP		46)
(define-inline-constant IPPROTO_GRE		47)
(define-inline-constant IPPROTO_ESP		50)
(define-inline-constant IPPROTO_AH		51)
(define-inline-constant IPPROTO_ICMPV6		58)
(define-inline-constant IPPROTO_NONE		59)
(define-inline-constant IPPROTO_DSTOPTS		60)
(define-inline-constant IPPROTO_MTP		92)
(define-inline-constant IPPROTO_ENCAP		98)
(define-inline-constant IPPROTO_PIM		103)
(define-inline-constant IPPROTO_COMP		108)
(define-inline-constant IPPROTO_SCTP		132)
(define-inline-constant IPPROTO_UDPLITE		136)
(define-inline-constant IPPROTO_RAW		255)

;;; Constants from "netinet/in.h"
(define-inline-constant IPPORT_ECHO		7)
(define-inline-constant IPPORT_DISCARD		9)
(define-inline-constant IPPORT_SYSTAT		11)
(define-inline-constant IPPORT_DAYTIME		13)
(define-inline-constant IPPORT_NETSTAT		15)
(define-inline-constant IPPORT_FTP		21)
(define-inline-constant IPPORT_TELNET		23)
(define-inline-constant IPPORT_SMTP		25)
(define-inline-constant IPPORT_TIMESERVER	37)
(define-inline-constant IPPORT_NAMESERVER	42)
(define-inline-constant IPPORT_WHOIS		43)
(define-inline-constant IPPORT_MTP		57)
(define-inline-constant IPPORT_TFTP		69)
(define-inline-constant IPPORT_RJE		77)
(define-inline-constant IPPORT_FINGER		79)
(define-inline-constant IPPORT_TTYLINK		87)
(define-inline-constant IPPORT_SUPDUP		95)
(define-inline-constant IPPORT_EXECSERVER	512)
(define-inline-constant IPPORT_LOGINSERVER	513)
(define-inline-constant IPPORT_CMDSERVER	514)
(define-inline-constant IPPORT_EFSSERVER	520)
(define-inline-constant IPPORT_BIFFUDP		512)
(define-inline-constant IPPORT_WHOSERVER	513)
(define-inline-constant IPPORT_ROUTESERVER	520)
;;These have been already defined above.
;;
;;(define-inline-constant IPPORT_RESERVED		1024)
;;(define-inline-constant IPPORT_USERRESERVED	5000)

;;; Constants from "netinet/in.h"
(define-inline-constant IN_CLASSA_NET		4278190080)
(define-inline-constant IN_CLASSA_NSHIFT	24)
(define-inline-constant IN_CLASSA_HOST		16777215)
(define-inline-constant IN_CLASSA_MAX		128)
(define-inline-constant IN_CLASSB_NET		4294901760)
(define-inline-constant IN_CLASSB_NSHIFT	16)
(define-inline-constant IN_CLASSB_HOST		65535)
(define-inline-constant IN_CLASSB_MAX		65536)
(define-inline-constant IN_CLASSC_NET		4294967040)
(define-inline-constant IN_CLASSC_NSHIFT	8)
(define-inline-constant IN_CLASSC_HOST		255)

;;; Constants from "netinet/tcp.h"
(define-inline-constant TCP_NODELAY		1)
(define-inline-constant TCP_MAXSEG		2)
(define-inline-constant TCP_CORK		3)
(define-inline-constant TCP_KEEPIDLE		4)
(define-inline-constant TCP_KEEPINTVL		5)
(define-inline-constant TCP_KEEPCNT		6)
(define-inline-constant TCP_SYNCNT		7)
(define-inline-constant TCP_LINGER2		8)
(define-inline-constant TCP_DEFER_ACCEPT	9)
(define-inline-constant TCP_WINDOW_CLAMP	10)
(define-inline-constant TCP_INFO		11)
(define-inline-constant TCP_QUICKACK		12)
(define-inline-constant TCP_CONGESTION		13)
(define-inline-constant TCP_MD5SIG		14)
(define-inline-constant TCP_ESTABLISHED		1)
(define-inline-constant TCP_SYN_SENT		2)
(define-inline-constant TCP_SYN_RECV		3)
(define-inline-constant TCP_FIN_WAIT1		4)
(define-inline-constant TCP_FIN_WAIT2		5)
(define-inline-constant TCP_TIME_WAIT		6)
(define-inline-constant TCP_CLOSE		7)
(define-inline-constant TCP_CLOSE_WAIT		8)
(define-inline-constant TCP_LAST_ACK		9)
(define-inline-constant TCP_LISTEN		10)
(define-inline-constant TCP_CLOSING		11)
(define-inline-constant TCPOPT_EOL		0)
(define-inline-constant TCPOPT_NOP		1)
(define-inline-constant TCPOPT_MAXSEG		2)
(define-inline-constant TCPOLEN_MAXSEG		4)
(define-inline-constant TCPOPT_WINDOW		3)
(define-inline-constant TCPOLEN_WINDOW		3)
(define-inline-constant TCPOPT_SACK_PERMITTED	4)
(define-inline-constant TCPOLEN_SACK_PERMITTED	2)
(define-inline-constant TCPOPT_SACK		5)
(define-inline-constant TCPOPT_TIMESTAMP	8)
(define-inline-constant TCPOLEN_TIMESTAMP	10)
(define-inline-constant TCPOLEN_TSTAMP_APPA	12)
(define-inline-constant TCPOPT_TSTAMP_HDR	16844810)
(define-inline-constant TCP_MSS			512)
(define-inline-constant TCP_MAXWIN		65535)
(define-inline-constant TCP_MAX_WINSHIFT	14)
(define-inline-constant SOL_TCP			6)
(define-inline-constant TCPI_OPT_TIMESTAMPS	1)
(define-inline-constant TCPI_OPT_SACK		2)
(define-inline-constant TCPI_OPT_WSCALE		4)
(define-inline-constant TCPI_OPT_ECN		8)
(define-inline-constant TCP_CA_Open		0)
(define-inline-constant TCP_CA_Disorder		1)
(define-inline-constant TCP_CA_CWR		2)
(define-inline-constant TCP_CA_Recovery		3)
(define-inline-constant TCP_CA_Loss		4)
(define-inline-constant TCP_MD5SIG_MAXKEYLEN	80)

;;; Constants from "netinet/udp.h"
(define-inline-constant UDP_CORK			1)
(define-inline-constant UDP_ENCAP			100)
(define-inline-constant UDP_ENCAP_ESPINUDP_NON_IKE	1)
(define-inline-constant UDP_ENCAP_ESPINUDP		2)
(define-inline-constant UDP_ENCAP_L2TPINUDP		3)
(define-inline-constant SOL_UDP				17)

;;; Constants from "linux/icmp.h"
(define-inline-constant ICMP_ECHOREPLY			0)
(define-inline-constant ICMP_DEST_UNREACH		3)
(define-inline-constant ICMP_SOURCE_QUENCH		4)
(define-inline-constant ICMP_REDIRECT			5)
(define-inline-constant ICMP_ECHO			8)
(define-inline-constant ICMP_TIME_EXCEEDED		11)
(define-inline-constant ICMP_PARAMETERPROB		12)
(define-inline-constant ICMP_TIMESTAMP			13)
(define-inline-constant ICMP_TIMESTAMPREPLY		14)
(define-inline-constant ICMP_INFO_REQUEST		15)
(define-inline-constant ICMP_INFO_REPLY			16)
(define-inline-constant ICMP_ADDRESS			17)
(define-inline-constant ICMP_ADDRESSREPLY		18)
(define-inline-constant NR_ICMP_TYPES			18)
(define-inline-constant ICMP_NET_UNREACH		0)
(define-inline-constant ICMP_HOST_UNREACH		1)
(define-inline-constant ICMP_PROT_UNREACH		2)
(define-inline-constant ICMP_PORT_UNREACH		3)
(define-inline-constant ICMP_FRAG_NEEDED		4)
(define-inline-constant ICMP_SR_FAILED			5)
(define-inline-constant ICMP_NET_UNKNOWN		6)
(define-inline-constant ICMP_HOST_UNKNOWN		7)
(define-inline-constant ICMP_HOST_ISOLATED		8)
(define-inline-constant ICMP_NET_ANO			9)
(define-inline-constant ICMP_HOST_ANO			10)
(define-inline-constant ICMP_NET_UNR_TOS		11)
(define-inline-constant ICMP_HOST_UNR_TOS		12)
(define-inline-constant ICMP_PKT_FILTERED		13)
(define-inline-constant ICMP_PREC_VIOLATION		14)
(define-inline-constant ICMP_PREC_CUTOFF		15)
(define-inline-constant NR_ICMP_UNREACH			15)
(define-inline-constant ICMP_REDIR_NET			0)
(define-inline-constant ICMP_REDIR_HOST			1)
(define-inline-constant ICMP_REDIR_NETTOS		2)
(define-inline-constant ICMP_REDIR_HOSTTOS		3)
(define-inline-constant ICMP_EXC_TTL			0)
(define-inline-constant ICMP_EXC_FRAGTIME		1)
(define-inline-constant ICMP_FILTER			1)

;;; Constants from "netinet/igmp.h"
(define-inline-constant IGMP_MINLEN			8)
(define-inline-constant IGMP_MEMBERSHIP_QUERY		17)
(define-inline-constant IGMP_V1_MEMBERSHIP_REPORT	18)
(define-inline-constant IGMP_V2_MEMBERSHIP_REPORT	22)
(define-inline-constant IGMP_V2_LEAVE_GROUP		23)
(define-inline-constant IGMP_DVMRP			19)
(define-inline-constant IGMP_PIM			20)
(define-inline-constant IGMP_TRACE			21)
(define-inline-constant IGMP_MTRACE_RESP		30)
(define-inline-constant IGMP_MTRACE			31)
(define-inline-constant IGMP_MAX_HOST_REPORT_DELAY	10)
(define-inline-constant IGMP_TIMER_SCALE		10)
(define-inline-constant IGMP_DELAYING_MEMBER		1)
(define-inline-constant IGMP_IDLE_MEMBER		2)
(define-inline-constant IGMP_LAZY_MEMBER		3)
(define-inline-constant IGMP_SLEEPING_MEMBER		4)
(define-inline-constant IGMP_AWAKENING_MEMBER		5)
(define-inline-constant IGMP_v1_ROUTER			1)
(define-inline-constant IGMP_v2_ROUTER			2)


;;;; h_errno friendly interface

(define-syntax h_errno-code
  (lambda (stx)
    (syntax-case stx ()
      ((_ ?code)
       (identifier? #'?code)
       (let ((sym (syntax->datum #'?code)))
	 (define them '(HOST_NOT_FOUND TRY_AGAIN NO_RECOVERY NO_ADDRESS))
	 (if (memq sym them)
	     #'?code
	   (syntax-violation 'h_errno-code "invalid symbol as h_errno code" (syntax->datum stx) sym)))))))

(define-syntax case-h_errno
  (syntax-rules (else)
    ((_ ?h_errno ((?code0 ?code ...) . ?body) ... (else . ?else-body))
     (let ((h_errno ?h_errno))
       (cond ((or (and (fixnum? ?code0) ($fx= h_errno ?code0))
		  (and (fixnum? ?code)  ($fx= h_errno ?code))
		  ...)
	      . ?body)
	     ...
	     (else . ?else-body))))
    ((_ ?h_errno ((?code0 ?code ...) . ?body) ...)
     (let ((h_errno ?h_errno))
       (cond ((or (and (fixnum? ?code0) ($fx= h_errno ?code0))
		  (and (fixnum? ?code)  ($fx= h_errno ?code))
		  ...)
	      . ?body)
	     ...)))
    ))


;;;; h_errno codes

(define-inline-constant HOST_NOT_FOUND	(neg 1))
(define-inline-constant TRY_AGAIN	(neg 2))
(define-inline-constant NO_RECOVERY	(neg 3))
(define-inline-constant NO_ADDRESS	(neg 4))


;;;; interprocess signals

(define-inline-constant NSIG		65)

;;; program error
(define-inline-constant SIGFPE		8)
(define-inline-constant SIGILL		4)
(define-inline-constant SIGSEGV		11)
(define-inline-constant SIGBUS		7)
(define-inline-constant SIGABRT		6)
(define-inline-constant SIGIOT		6)
(define-inline-constant SIGTRAP		5)
(define-inline-constant SIGEMT		#f)
(define-inline-constant SIGSYS		31)

;;; termination
(define-inline-constant SIGTERM		15)
(define-inline-constant SIGINT		2)
(define-inline-constant SIGQUIT		3)
(define-inline-constant SIGKILL		9)
(define-inline-constant SIGHUP		1)

;;; alarm
(define-inline-constant SIGALRM		14)
(define-inline-constant SIGVRALRM	#f)
(define-inline-constant SIGPROF		27)

;;; asynchronous I/O
(define-inline-constant SIGIO		29)
(define-inline-constant SIGURG		23)
(define-inline-constant SIGPOLL		29)

;;; job control
(define-inline-constant SIGCHLD		17)
(define-inline-constant SIGCLD		17)
(define-inline-constant SIGCONT		18)
(define-inline-constant SIGSTOP		19)
(define-inline-constant SIGTSTP		20)
(define-inline-constant SIGTTIN		21)
(define-inline-constant SIGTTOU		22)

;;; operation error
(define-inline-constant SIGPIPE		13)
(define-inline-constant SIGLOST		#f)
(define-inline-constant SIGXCPU		24)
(define-inline-constant SIGXSFZ		#f)

;;; misc
(define-inline-constant SIGUSR1		10)
(define-inline-constant SIGUSR2		12)
(define-inline-constant SIGWINCH	28)
(define-inline-constant SIGINFO		#f)


;;;; done

)

;;; end of file
;; Local Variables:
;; mode: scheme
;; End:
