;;; -*- coding: utf-8-unix -*-
;;;
;;;Part of: Vicare
;;;Contents: utility functions for fixed-size words
;;;Date: Sun Oct 23, 2011
;;;
;;;Abstract
;;;
;;;	This library  is both installed and  used when expanding Vicare's  own source
;;;	code.   For this  reason it  must export  only: bindings  imported by  Vicare
;;;	itself, syntaxes whose  expansion reference only bindings  imported by Vicare
;;;	itself.
;;;
;;;Copyright (C) 2011-2013, 2015, 2016, 2017 Marco Maggi <marco.maggi-ipsu@poste.it>
;;;
;;;This program is free software: you can  redistribute it and/or modify it under the
;;;terms  of  the GNU  General  Public  License as  published  by  the Free  Software
;;;Foundation,  either version  3  of the  License,  or (at  your  option) any  later
;;;version.
;;;
;;;This program is  distributed in the hope  that it will be useful,  but WITHOUT ANY
;;;WARRANTY; without  even the implied warranty  of MERCHANTABILITY or FITNESS  FOR A
;;;PARTICULAR PURPOSE.  See the GNU General Public License for more details.
;;;
;;;You should have received a copy of  the GNU General Public License along with this
;;;program.  If not, see <http://www.gnu.org/licenses/>.
;;;


#!vicare
(library (vicare platform words (0 4 2015 5 28))
  (options typed-language)
  (export

    ;; syntaxes
    case-word-size	case-endianness

    ;; auxiliary syntaxes
    big				little

    ;; sizes
    SIZEOF_CHAR
    SIZEOF_SHORT	SIZEOF_INT
    SIZEOF_LONG		SIZEOF_LONG_LONG
    SIZEOF_SIZE_T	SIZEOF_SSIZE_T
    SIZEOF_OFF_T	SIZEOF_PTRDIFF_T
    SIZEOF_FLOAT	SIZEOF_DOUBLE
    SIZEOF_DOUBLE_COMPLEX
    SIZEOF_POINTER

    ;; predicates
    word?
    word-u8?		word-s8?
    word-u16?		word-s16?
    word-u32?		word-s32?
    word-u64?		word-s64?
    word-u128?		word-s128?
    word-u256?		word-s256?
    machine-word?

    unsigned-char?	signed-char?
    unsigned-short?	signed-short?
    unsigned-int?	signed-int?
    unsigned-long?	signed-long?
    unsigned-long-long?	signed-long-long?
    pointer-integer?

    size_t?		ssize_t?
    off_t?		ptrdiff_t?

    ;; sign predicates
    positive-word-s8?			non-positive-word-s8?
    negative-word-s8?			non-negative-word-s8?
    positive-word-s16?			non-positive-word-s16?
    negative-word-s16?			non-negative-word-s16?
    positive-word-s32?			non-positive-word-s32?
    negative-word-s32?			non-negative-word-s32?
    positive-word-s64?			non-positive-word-s64?
    negative-word-s64?			non-negative-word-s64?
    positive-word-s128?			non-positive-word-s128?
    negative-word-s128?			non-negative-word-s128?
    positive-word-s256?			non-positive-word-s256?
    negative-word-s256?			non-negative-word-s256?

    positive-signed-char?		non-positive-signed-char?
    negative-signed-char?		non-negative-signed-char?
    positive-signed-int?		non-positive-signed-int?
    negative-signed-int?		non-negative-signed-int?
    positive-signed-long?		non-positive-signed-long?
    negative-signed-long?		non-negative-signed-long?
    positive-signed-long-long?		non-positive-signed-long-long?
    negative-signed-long-long?		non-negative-signed-long-long?
    positive-ssize_t?			non-positive-ssize_t?
    negative-ssize_t?			non-negative-ssize_t?
    positive-off_t?			non-positive-off_t?
    negative-off_t?			non-negative-off_t?
    positive-ptrdiff_t?			non-positive-ptrdiff_t?
    negative-ptrdiff_t?			non-negative-ptrdiff_t?

    ;; inclusive limits
    greatest-u8		least-u8
    greatest-s8		least-s8
    greatest-u16	least-u16
    greatest-s16	least-s16
    greatest-u32	least-u32
    greatest-s32	least-s32
    greatest-u64	least-u64
    greatest-s64	least-s64
    greatest-u128	least-u128
    greatest-s128	least-s128
    greatest-u256	least-u256
    greatest-s256	least-s256
    greatest-machine-word
    least-machine-word

    greatest-c-unsigned-char		greatest-c-signed-char
    least-c-unsigned-char		least-c-signed-char
    greatest-c-unsigned-short		greatest-c-signed-short
    least-c-unsigned-short		least-c-signed-short
    greatest-c-unsigned-int		greatest-c-signed-int
    least-c-unsigned-int		least-c-signed-int
    greatest-c-unsigned-long		greatest-c-signed-long
    least-c-unsigned-long		least-c-signed-long
    greatest-c-unsigned-long-long	greatest-c-signed-long-long
    least-c-unsigned-long-long		least-c-signed-long-long

    greatest-c-pointer			least-c-pointer

    greatest-c-size_t			least-c-size_t
    greatest-c-ssize_t			least-c-ssize_t
    greatest-c-off_t			least-c-off_t
    greatest-c-ptrdiff_t		least-c-ptrdiff_t

    ;; exclusive limits
    greatest-u8*	least-u8*
    greatest-s8*	least-s8*
    greatest-u16*	least-u16*
    greatest-s16*	least-s16*
    greatest-u32*	least-u32*
    greatest-s32*	least-s32*
    greatest-u64*	least-u64*
    greatest-s64*	least-s64*
    greatest-u128*	least-u128*
    greatest-s128*	least-s128*
    greatest-u256*	least-u256*
    greatest-s256*	least-s256*
    greatest-machine-word*
    least-machine-word*

    greatest-c-unsigned-char*		greatest-c-signed-char*
    least-c-unsigned-char*		least-c-signed-char*
    greatest-c-unsigned-short*		greatest-c-signed-short*
    least-c-unsigned-short*		least-c-signed-short*
    greatest-c-unsigned-int*		greatest-c-signed-int*
    least-c-unsigned-int*		least-c-signed-int*
    greatest-c-unsigned-long*		greatest-c-signed-long*
    least-c-unsigned-long*		least-c-signed-long*
    greatest-c-unsigned-long-long*	greatest-c-signed-long-long*
    least-c-unsigned-long-long*		least-c-signed-long-long*

    greatest-c-pointer*			least-c-pointer*

    greatest-c-size_t*			least-c-size_t*
    greatest-c-ssize_t*			least-c-ssize_t*
    greatest-c-off_t*			least-c-off_t*
    greatest-c-ptrdiff_t*		least-c-ptrdiff_t*

    ;; fixnum alignment
    fixnum-aligned-to-2?
    fixnum-aligned-to-4?
    fixnum-aligned-to-8?
    fixnum-aligned-to?

    #| end of EXPORT |# )
  (import (vicare)
    (vicare system $fx)
    (vicare system $bignums)
    (for (prefix (vicare platform configuration)
		 config::)
      expand))


;;;; helpers

(define-syntax case-data-size
  (syntax-rules ()
    ((case-data-size 1
       ((1)	. ?body-1)
       ((2)	. ?body-2)
       ((4)	. ?body-4)
       ((8)	. ?body-8))
     (begin . ?body-1))
    ((case-data-size 2
       ((1)	. ?body-1)
       ((2)	. ?body-2)
       ((4)	. ?body-4)
       ((8)	. ?body-8))
     (begin . ?body-2))
    ((case-data-size 4
       ((1)	. ?body-1)
       ((2)	. ?body-2)
       ((4)	. ?body-4)
       ((8)	. ?body-8))
     (begin . ?body-4))
    ((case-data-size 8
       ((1)	. ?body-1)
       ((2)	. ?body-2)
       ((4)	. ?body-4)
       ((8)	. ?body-8))
     (begin . ?body-8))))

;;; --------------------------------------------------------------------

(define-syntax-rule (declare-positive-predicate ?who ?pred)
  (define-inline (?who obj)
    (and (?pred obj)
	 (positive? obj))))

(define-syntax-rule (declare-negative-predicate ?who ?pred)
  (define-inline (?who obj)
    (and (?pred obj)
	 (negative? obj))))

(define-syntax-rule (declare-non-positive-predicate ?who ?pred)
  (define-inline (?who obj)
    (and (?pred obj)
	 (non-positive? obj))))

(define-syntax-rule (declare-non-negative-predicate ?who ?pred)
  (define-inline (?who obj)
    (and (?pred obj)
	 (non-negative? obj))))

(define-syntax (declare-sign-predicates stx)
  (define* (identifier-prefix prefix {id identifier?})
    (datum->syntax id (string->symbol (string-append
				       (cond ((string? prefix)
					      prefix)
					     ((symbol? prefix)
					      (symbol->string prefix))
					     ((identifier? prefix)
					      (symbol->string (syntax->datum prefix)))
					     (else
					      (assertion-violation __who__
						"expected string, symbol or identifier as prefix argument" prefix)))
				       (symbol->string (syntax->datum id))))))
  (syntax-case stx ()
    ((_ ?pred)
     (with-syntax
	 ((POSITIVE-WHO		(identifier-prefix "positive-"     #'?pred))
	  (NEGATIVE-WHO		(identifier-prefix "negative-"     #'?pred))
	  (NON-POSITIVE-WHO	(identifier-prefix "non-positive-" #'?pred))
	  (NON-NEGATIVE-WHO	(identifier-prefix "non-negative-" #'?pred)))
       #'(begin
	   (declare-positive-predicate POSITIVE-WHO ?pred)
	   (declare-negative-predicate NEGATIVE-WHO ?pred)
	   (declare-non-positive-predicate NON-POSITIVE-WHO ?pred)
	   (declare-non-negative-predicate NON-NEGATIVE-WHO ?pred))))
    ))


;;;; syntaxes

(define-syntax case-word-size
  (if (= 4 config::wordsize)
      (syntax-rules ()
	((_ ((32) . ?body-32) ((64) . ?body-64))
	 (begin . ?body-32)))
    (syntax-rules ()
      ((_ ((32) . ?body-32) ((64) . ?body-64))
       (begin . ?body-64)))))

(define-syntax big	(syntax-rules ()))
(define-syntax little	(syntax-rules ()))

(define-syntax case-endianness
  (lambda (stx)
    (syntax-case stx (big little)
      ((case-endianness (?who ?endianness)
	 ((little)	. ?lit-body)
	 ((big)		. ?big-body))
       (and (identifier? #'?who)
	    (identifier? #'?endianness))
       #'(case-endianness (?who ?endianness)
	   ((big)	. ?big-body)
	   ((little)	. ?lit-body)))

      ((case-endianness (?who ?endianness)
	 ((big)		. ?big-body)
	 ((little)	. ?lit-body))
       (and (identifier? #'?who)
	    (identifier? #'?endianness))
       #'(case ?endianness
	   ((big)	. ?big-body)
	   ((little)	. ?lit-body)
	   (else
	    (assertion-violation ?who "expected endianness symbol as argument" ?endianness)))))))


;;;; constants

(define-inline-constant U8MAX		255)
(define-inline-constant U8MIN		0)
(define-inline-constant S8MAX		+127)
(define-inline-constant S8MIN		-128)

(define-inline-constant U16MAX		65535)		#;(- (expt 2 16) 1)
(define-inline-constant U16MIN		0)
(define-inline-constant S16MAX		+32767)		#;(- (expt 2 15) 1)
(define-inline-constant S16MIN		-32768)		#;(- (expt 2 15))

(define-inline-constant U32MAX		4294967295)	#;(- (expt 2 32) 1)
(define-inline-constant U32MIN		0)
(define-inline-constant S32MAX		+2147483647)	#;(- (expt 2 31) 1)
(define-inline-constant S32MIN		-2147483648)	#;(- (expt 2 31))

(define-inline-constant U64MAX		18446744073709551615)	#;(- (expt 2 64) 1)
(define-inline-constant U64MIN		0)
(define-inline-constant S64MAX		+9223372036854775807)	#;(- (expt 2 63) 1)
(define-inline-constant S64MIN		-9223372036854775808)	#;(- (expt 2 63))

(define-inline-constant U128MAX
  340282366920938463463374607431768211455) #;(- (expt 2 128) 1)
(define-inline-constant U128MIN
  0)
(define-inline-constant S128MAX
  170141183460469231731687303715884105727) #;(- (expt 2 127) 1)
(define-inline-constant S128MIN
  -170141183460469231731687303715884105728) #;(- (expt 2 127))

(define-inline-constant U256MAX
  #;(- (expt 2 256) 1)
  115792089237316195423570985008687907853269984665640564039457584007913129639935)
(define-inline-constant U256MIN
  0)
(define-inline-constant S256MAX
  #;(- (expt 2 255) 1)
  57896044618658097711785492504343953926634992332820282019728792003956564819967)
(define-inline-constant S256MIN
  #;(- (expt 2 255))
  -57896044618658097711785492504343953926634992332820282019728792003956564819968)

;;; --------------------------------------------------------------------

(define-inline-constant U8MAX*		256)
(define-inline-constant U8MIN*		-1)
(define-inline-constant S8MAX*		+128)
(define-inline-constant S8MIN*		-129)

(define-inline-constant U16MAX*		65536)
(define-inline-constant U16MIN*		-1)
(define-inline-constant S16MAX*		+32768)
(define-inline-constant S16MIN*		-32769)

(define-inline-constant U32MAX*		4294967296)
(define-inline-constant U32MIN*		-1)
(define-inline-constant S32MAX*		+2147483648)
(define-inline-constant S32MIN*		-2147483649)

(define-inline-constant U64MAX*		18446744073709551616)
(define-inline-constant U64MIN*		-1)
(define-inline-constant S64MAX*		+9223372036854775808)
(define-inline-constant S64MIN*		-9223372036854775809)

(define-inline-constant U128MAX*	340282366920938463463374607431768211456)
(define-inline-constant U128MIN*	-1)
(define-inline-constant S128MAX*	170141183460469231731687303715884105728)
(define-inline-constant S128MIN*	-170141183460469231731687303715884105729)

(define-inline-constant U256MAX*
  115792089237316195423570985008687907853269984665640564039457584007913129639936)
(define-inline-constant U256MIN*
  -1)
(define-inline-constant S256MAX*
  57896044618658097711785492504343953926634992332820282019728792003956564819968)
(define-inline-constant S256MIN*
  -57896044618658097711785492504343953926634992332820282019728792003956564819969)

;;; --------------------------------------------------------------------

(define-inline-constant SIZEOF_CHAR		1)
(define-inline-constant SIZEOF_SHORT		2)
(define-inline-constant SIZEOF_INT		4)
(define-inline-constant SIZEOF_LONG		8)
(define-inline-constant SIZEOF_LONG_LONG	8)
(define-inline-constant SIZEOF_SIZE_T		8)
(define-inline-constant SIZEOF_SSIZE_T		8)
(define-inline-constant SIZEOF_OFF_T		8)
(define-inline-constant SIZEOF_PTRDIFF_T	8)
(define-inline-constant SIZEOF_FLOAT		4)
(define-inline-constant SIZEOF_DOUBLE		8)
(define-inline-constant SIZEOF_DOUBLE_COMPLEX	16)
(define-inline-constant SIZEOF_POINTER		8)

;;; --------------------------------------------------------------------

(define-inline-constant SCHAR_MIN		-128)
(define-inline-constant SCHAR_MAX		127)
(define-inline-constant UCHAR_MAX		255)
(define-inline-constant CHAR_MIN		-128)
(define-inline-constant CHAR_MAX		127)
(define-inline-constant SHRT_MIN		-32768)
(define-inline-constant SHRT_MAX		32767)
(define-inline-constant USHRT_MAX		65535)
(define-inline-constant INT_MIN			-2147483648)
(define-inline-constant INT_MAX			2147483647)
(define-inline-constant UINT_MAX		4294967295)
(define-inline-constant LONG_MIN		-9223372036854775808)
(define-inline-constant LONG_MAX		9223372036854775807)
(define-inline-constant ULONG_MAX		18446744073709551615)
;; If the "long  long" constants are undefined, assume  that "long long"
;; is a 64-bit integer.
(define-inline-constant LONG_LONG_MIN		(or -9223372036854775808
						    ;; (- (expt 2 63))
						    -9223372036854775808))
(define-inline-constant LONG_LONG_MAX		(or 9223372036854775807
						    ;; (- (expt 2 63) 1)
						    +9223372036854775807))
(define-inline-constant ULONG_LONG_MAX		(or 18446744073709551615
						    ;; (- (expt 2 64) 1)
						    18446744073709551615))
(define-inline-constant WCHAR_MAX		2147483647)

;;; --------------------------------------------------------------------

(define-syntax SIZE_T_MAX
  (if (= 4 8)
      (identifier-syntax U32MAX)
    (identifier-syntax U64MAX)))

(define-inline-constant SIZE_T_MIN	0)

(define-syntax SSIZE_T_MAX
  (if (= 4 8)
      (identifier-syntax S32MAX)
    (identifier-syntax S64MAX)))

(define-syntax SSIZE_T_MIN
  (if (= 4 8)
      (identifier-syntax S32MIN)
    (identifier-syntax S64MIN)))

(define-syntax OFF_T_MAX
  (if (= 4 8)
      (identifier-syntax S32MAX)
    (identifier-syntax S64MAX)))

(define-syntax OFF_T_MIN
  (if (= 4 8)
      (identifier-syntax S32MIN)
    (identifier-syntax S64MIN)))

(define-syntax PTRDIFF_T_MAX
  (if (= 4 8)
      (identifier-syntax S32MAX)
    (identifier-syntax S64MAX)))

(define-syntax PTRDIFF_T_MIN
  (if (= 4 8)
      (identifier-syntax S32MIN)
    (identifier-syntax S64MIN)))


(define-syntax-rule (%bnbncmp X Y fxcmp)
  (fxcmp (foreign-call "ikrt_bnbncomp" X Y) 0))

(define-inline ($bnbn= X Y)
  (%bnbncmp X Y $fx=))

(define-inline ($bnbn< X Y)
  (%bnbncmp X Y $fx<))

(define-inline ($bnbn> X Y)
  (%bnbncmp X Y $fx>))

(define-inline ($bnbn<= X Y)
  (%bnbncmp X Y $fx<=))

(define-inline ($bnbn>= X Y)
  (%bnbncmp X Y $fx>=))


;;;; limits

(define-inline (greatest-u8)		U8MAX)
(define-inline (greatest-s8)		S8MAX)
(define-inline (least-u8)		U8MIN)
(define-inline (least-s8)		S8MIN)

(define-inline (greatest-u16)		U16MAX)
(define-inline (greatest-s16)		S16MAX)
(define-inline (least-u16)		U16MIN)
(define-inline (least-s16)		S16MIN)

(define-inline (greatest-u32)		U32MAX)
(define-inline (greatest-s32)		S32MAX)
(define-inline (least-u32)		U32MIN)
(define-inline (least-s32)		S32MIN)

(define-inline (greatest-u64)		U64MAX)
(define-inline (greatest-s64)		S64MAX)
(define-inline (least-u64)		U64MIN)
(define-inline (least-s64)		S64MIN)

(define-inline (greatest-u128)		U128MAX)
(define-inline (greatest-s128)		S128MAX)
(define-inline (least-u128)		U128MIN)
(define-inline (least-s128)		S128MIN)

(define-inline (greatest-u256)		U256MAX)
(define-inline (greatest-s256)		S256MAX)
(define-inline (least-u256)		U256MIN)
(define-inline (least-s256)		S256MIN)

;;; --------------------------------------------------------------------

(define-inline (greatest-machine-word)
  (case-word-size
   ((32)	(greatest-u32))
   ((64)	(greatest-u64))))

(define-inline (least-machine-word)
  (case-word-size
   ((32)	(least-u32))
   ((64)	(least-u64))))

;;; --------------------------------------------------------------------

(define-inline (greatest-c-unsigned-char)	UCHAR_MAX)
(define-inline (greatest-c-signed-char)		CHAR_MAX)
(define-inline (least-c-unsigned-char)		0)
(define-inline (least-c-signed-char)		CHAR_MIN)

(define-inline (greatest-c-unsigned-short)	USHRT_MAX)
(define-inline (greatest-c-signed-short)	SHRT_MAX)
(define-inline (least-c-unsigned-short)		0)
(define-inline (least-c-signed-short)		SHRT_MIN)

(define-inline (greatest-c-unsigned-int)	UINT_MAX)
(define-inline (greatest-c-signed-int)		INT_MAX)
(define-inline (least-c-unsigned-int)		0)
(define-inline (least-c-signed-int)		INT_MIN)

(define-inline (greatest-c-unsigned-long)	ULONG_MAX)
(define-inline (greatest-c-signed-long)		LONG_MAX)
(define-inline (least-c-unsigned-long)		0)
(define-inline (least-c-signed-long)		LONG_MIN)

(define-inline (greatest-c-unsigned-long-long)	ULONG_LONG_MAX)
(define-inline (greatest-c-signed-long-long)	LONG_LONG_MAX)
(define-inline (least-c-unsigned-long-long)	0)
(define-inline (least-c-signed-long-long)	LONG_LONG_MIN)

(define-inline (greatest-c-pointer)
  (case-word-size
   ((32)	U32MAX)
   ((64)	U64MAX)))

(define-inline (least-c-pointer)
  0)

(define-inline (greatest-c-size_t)		SIZE_T_MAX)
(define-inline (greatest-c-ssize_t)		SSIZE_T_MAX)
(define-inline (least-c-size_t)			0)
(define-inline (least-c-ssize_t)		SSIZE_T_MIN)
(define-inline (greatest-c-off_t)		OFF_T_MAX)
(define-inline (least-c-off_t)			OFF_T_MIN)
(define-inline (greatest-c-ptrdiff_t)		PTRDIFF_T_MAX)
(define-inline (least-c-ptrdiff_t)		PTRDIFF_T_MIN)

;;; --------------------------------------------------------------------

(define-inline (greatest-u8*)		U8MAX*)
(define-inline (greatest-s8*)		S8MAX*)
(define-inline (least-u8*)		U8MIN*)
(define-inline (least-s8*)		S8MIN*)

(define-inline (greatest-u16*)		U16MAX*)
(define-inline (greatest-s16*)		S16MAX*)
(define-inline (least-u16*)		U16MIN*)
(define-inline (least-s16*)		S16MIN*)

(define-inline (greatest-u32*)		U32MAX*)
(define-inline (greatest-s32*)		S32MAX*)
(define-inline (least-u32*)		U32MIN*)
(define-inline (least-s32*)		S32MIN*)

(define-inline (greatest-u64*)		U64MAX*)
(define-inline (greatest-s64*)		S64MAX*)
(define-inline (least-u64*)		U64MIN*)
(define-inline (least-s64*)		S64MIN*)

(define-inline (greatest-u128*)		U128MAX*)
(define-inline (greatest-s128*)		S128MAX*)
(define-inline (least-u128*)		U128MIN*)
(define-inline (least-s128*)		S128MIN*)

(define-inline (greatest-u256*)		U256MAX*)
(define-inline (greatest-s256*)		S256MAX*)
(define-inline (least-u256*)		U256MIN*)
(define-inline (least-s256*)		S256MIN*)

;;; --------------------------------------------------------------------

(define-inline (greatest-machine-word*)
  (case-word-size
   ((32)	(greatest-u32*))
   ((64)	(greatest-u64*))))

(define-inline (least-machine-word*)
  (case-word-size
   ((32)	(least-u32*))
   ((64)	(least-u64*))))

;;; --------------------------------------------------------------------

(define-inline (greatest-c-unsigned-char*)	(+ (greatest-c-unsigned-char) 1))
(define-inline (greatest-c-signed-char*)	(+ (greatest-c-signed-char) 1))
(define-inline (least-c-unsigned-char*)		(- (least-c-unsigned-char) 1))
(define-inline (least-c-signed-char*)		(- (least-c-signed-char) 1))

(define-inline (greatest-c-unsigned-short*)	(+ (greatest-c-unsigned-short) 1))
(define-inline (greatest-c-signed-short*)	(+ (greatest-c-signed-short) 1))
(define-inline (least-c-unsigned-short*)	(- (least-c-unsigned-short) 1))
(define-inline (least-c-signed-short*)		(- (least-c-signed-short) 1))

(define-inline (greatest-c-unsigned-int*)	(+ (greatest-c-unsigned-int) 1))
(define-inline (greatest-c-signed-int*)		(+ (greatest-c-signed-int) 1))
(define-inline (least-c-unsigned-int*)		(- (least-c-unsigned-int) 1))
(define-inline (least-c-signed-int*)		(- (least-c-signed-int) 1))

(define-inline (greatest-c-unsigned-long*)	(+ (greatest-c-unsigned-long) 1))
(define-inline (greatest-c-signed-long*)	(+ (greatest-c-signed-long) 1))
(define-inline (least-c-unsigned-long*)		(- (least-c-unsigned-long) 1))
(define-inline (least-c-signed-long*)		(- (least-c-signed-long) 1))

(define-inline (greatest-c-unsigned-long-long*)	(+ (greatest-c-unsigned-long-long) 1))
(define-inline (greatest-c-signed-long-long*)	(+ (greatest-c-signed-long-long) 1))
(define-inline (least-c-unsigned-long-long*)	(- (least-c-unsigned-long-long) 1))
(define-inline (least-c-signed-long-long*)	(- (least-c-signed-long-long) 1))

(define-inline (greatest-c-pointer*)
  (case-word-size
   ((32)	U32MAX*)
   ((64)	U64MAX*)))

(define-inline (least-c-pointer*)
  (case-word-size
   ((32)	U32MIN*)
   ((64)	U64MIN*)))

(define-inline (greatest-c-size_t*)		(+ SIZE_T_MAX 1))
(define-inline (least-c-size_t*)		-1)

(define-inline (greatest-c-ssize_t*)		(+ SSIZE_T_MAX 1))
(define-inline (least-c-ssize_t*)		(- SSIZE_T_MIN 1))

(define-inline (greatest-c-off_t*)		(+ OFF_T_MAX 1))
(define-inline (least-c-off_t*)			(- OFF_T_MIN 1))

(define-inline (greatest-c-ptrdiff_t*)		(+ PTRDIFF_T_MAX 1))
(define-inline (least-c-ptrdiff_t*)		(- PTRDIFF_T_MIN 1))


;;;; bit-sized word predicates

(define-inline (word? N)
  (or (fixnum? N) (bignum? N)))

(define-inline (machine-word? N)
  (case-word-size
   ((32) (word-u32? N))
   ((64) (word-u64? N))))

;;; --------------------------------------------------------------------

(define-inline (word-u8? N)
  (and (fixnum? N)
       ($fx>= N U8MIN)
       ($fx<= N U8MAX)))

(define-inline (word-s8? N)
  (and (fixnum? N)
       ($fx>= N S8MIN)
       ($fx<= N S8MAX)))

;;; --------------------------------------------------------------------

(define-inline (word-u16? N)
  (and (fixnum? N)
       ($fx>= N 0)
       ($fx<= N U16MAX)))

(define-inline (word-s16? N)
  (and (fixnum? N)
       ($fx>= N S16MIN)
       ($fx<= N S16MAX)))

;;; --------------------------------------------------------------------

(define-inline (word-u32? N)
  (case-word-size
   ((32) (if (fixnum? N)
	     ($fx<= 0 N)
	   (and (bignum? N)
		($bignum-positive? N)
		($bnbn<= N U32MAX))))
   ((64) (and (fixnum? N)
	      ($fx>= N 0)
	      ($fx<= N U32MAX)))))

(define-inline (word-s32? N)
  (case-word-size
   ((32) (or (fixnum? N)
	     (and (bignum? N)
		  ($bnbn>= N S32MIN)
		  ($bnbn<= N S32MAX))))
   ((64) (and (fixnum? N)
	      ($fx>= N S32MIN)
	      ($fx<= N S32MAX)))))

;;; --------------------------------------------------------------------

(define-inline (word-u64? N)
  (if (fixnum? N)
      ($fx<= 0 N)
    (and (bignum? N)
	 ($bignum-positive? N)
	 ($bnbn<= N U64MAX))))

(define-inline (word-s64? N)
  (or (fixnum? N)
      (and (bignum? N)
	   ($bnbn>= N S64MIN)
	   ($bnbn<= N S64MAX))))

;;; --------------------------------------------------------------------

(define-inline (word-u128? N)
  (if (fixnum? N)
      ($fx<= 0 N)
    (and (bignum? N)
	 ($bignum-positive? N)
	 ($bnbn<= N U128MAX))))

(define-inline (word-s128? N)
  (or (fixnum? N)
      (and (bignum? N)
	   ($bnbn>= N S128MIN)
	   ($bnbn<= N S128MAX))))

;;; --------------------------------------------------------------------

(define-inline (word-u256? N)
  (if (fixnum? N)
      ($fx<= 0 N)
    (and (bignum? N)
	 ($bignum-positive? N)
	 ($bnbn<= N U256MAX))))

(define-inline (word-s256? N)
  (or (fixnum? N)
      (and (bignum? N)
	   ($bnbn>= N S256MIN)
	   ($bnbn<= N S256MAX))))

;;; --------------------------------------------------------------------

(declare-sign-predicates word-s8?)
(declare-sign-predicates word-s16?)
(declare-sign-predicates word-s32?)
(declare-sign-predicates word-s64?)
(declare-sign-predicates word-s128?)
(declare-sign-predicates word-s256?)


;;;; C language data predicates

(define-auxiliary-syntaxes signed unsigned)
(let-syntax
    ((define-predicate (syntax-rules (signed unsigned)
			 ((_ ?who ?data-size unsigned)
			  (define-syntax (?who stx)
			    (syntax-case stx ()
			      ((_ ?N)
			       (case-data-size ?data-size
				 ((1)	#'(word-u8?  ?N))
				 ((2)	#'(word-u16? ?N))
				 ((4)	#'(word-u32? ?N))
				 ((8)	#'(word-u64? ?N))))
			      )))
			 ((_ ?who ?data-size signed)
			  (define-syntax (?who stx)
			    (syntax-case stx ()
			      ((_ ?N)
			       (case-data-size ?data-size
				 ((1)	#'(word-s8?  ?N))
				 ((2)	#'(word-s16? ?N))
				 ((4)	#'(word-s32? ?N))
				 ((8)	#'(word-s64? ?N))))
			      )))
			 )))
  (define-predicate unsigned-char?		1		unsigned)
  (define-predicate unsigned-short?		2		unsigned)
  (define-predicate unsigned-int?		4		unsigned)
  (define-predicate unsigned-long?		8		unsigned)
  (define-predicate unsigned-long-long?		8	unsigned)
  (define-predicate signed-char?		1		signed)
  (define-predicate signed-short?		2		signed)
  (define-predicate signed-int?			4		signed)
  (define-predicate signed-long?		8		signed)
  (define-predicate signed-long-long?		8	signed)

  (define-predicate pointer-integer?		8	unsigned)

  (define-predicate size_t?			8		unsigned)
  (define-predicate ssize_t?			8	signed)
  (define-predicate off_t?			8		signed)
  (define-predicate ptrdiff_t?			8	signed)
  #| end of LET-SYNTAX |# )

;;; -------------------------------------------------------------------- ;

(declare-sign-predicates signed-char?)
(declare-sign-predicates signed-int?)
(declare-sign-predicates signed-long?)
(declare-sign-predicates signed-long-long?)
(declare-sign-predicates ssize_t?)
(declare-sign-predicates off_t?)
(declare-sign-predicates ptrdiff_t?)


;;;; vector index alignment
;;
;;* If an index  is a multiple of 2 its least  significant bit is set to
;;zero:
;;
;;   (number->string  2 2)	=>     "10"
;;   (number->string  4 2)	=>    "100"
;;   (number->string  6 2)	=>    "110"
;;   (number->string  8 2)	=>   "1000"
;;   (number->string 10 2)	=>   "1010"
;;       ...
;;
;;so to test if an index is aligned to a multiple of 2 we just do:
;;
;;   ($fxzero? ($fxand index #b1))
;;
;;* If an index is a multiple  of 4 its least significant 2 bits are set
;;to zero:
;;
;;   (number->string  4 2)	=>    "100"
;;   (number->string  8 2)	=>   "1000"
;;   (number->string 12 2)	=>   "1100"
;;   (number->string 16 2)	=>  "10000"
;;       ...
;;
;;so to test if an index is aligned to a multiple of 4 we just do:
;;
;;   ($fxzero? ($fxand index #b11))
;;
;;* If an index is a multiple  of 8 its least significant 3 bits are set
;;to zero:
;;
;;   (number->string  8 2)	=>   "1000"
;;   (number->string 16 2)	=>  "10000"
;;   (number->string 24 2)	=>  "11000"
;;   (number->string 32 2)	=> "100000"
;;       ...
;;
;;so to test if an index is aligned to a multiple of 8 we just do:
;;
;;   ($fxzero? ($fxand index #b111))
;;
;;Notice that:
;;
;;   sizeof(float)	=> 4
;;   sizeof(double)	=> 8
;;

(define-inline (fixnum-aligned-to-2? N)
  ($fxzero? ($fxand #b1 N)))

(define-inline (fixnum-aligned-to-4? N)
  ($fxzero? ($fxand #b11 N)))

(define-inline (fixnum-aligned-to-8? N)
  ($fxzero? ($fxand #b111 N)))

(define-inline (fixnum-aligned-to? N alignment)
  ($fxzero? (fxmod N alignment)))


;;;; done

#| end of library |# )

;;; end of file
;; Local Variables:
;; eval: (put 'case-data-size 'scheme-indent-function 1)
;; End:
